@@ -1,4 +1,6 @@
-# vi:sw=4:et
+"""
+1:n and n:m relations in the data model are saved using a collection.
+"""
 
 import inspect
 
@@ -6,6 +8,9 @@ class CollectionError(Exception):
     pass
 
 class collection(object):
+    """
+    Collection (set-like) for model elements' 1:n and n:m relationships.
+    """
 
     def __init__(self, property, object, type):
         self.property = property
@@ -62,8 +67,10 @@ class collection(object):
 
 
     def index(self, key):
-        """Given an object, return the position of that object in the
-        collection."""
+        """
+        Given an object, return the position of that object in the
+        collection.
+        """
         return self.items.index(key)
 
 
@@ -191,25 +198,21 @@ class collection(object):
         return False
 
 
-    def moveUp(self, value):
-        """Move element up. Owner is notified about the change.
+    def swap(self, item1, item2):
         """
-        i1 = self.items.index(value)
-        i2 = i1 - 1
-        if i2 >= 0:
+        Swap two elements. Return true if swap was successful.
+        """
+        try:
+            i1 = self.items.index(item1)
+            i2 = self.items.index(item2)
             self.items[i1], self.items[i2] = self.items[i2], self.items[i1]
-            self.property.notify(self.object) # send a notification that this list has changed
-        else:
-            log.warning('Cannot move up first element')
+            # send a notification that this list has changed
+            self.property.notify(self.object)
+            return True
+        except IndexError, ex:
+            return False
+        except ValueError, ex:
+            return False
 
 
-    def moveDown(self, value):
-        """Move element down. Owner is notified about the change.
-        """
-        i1 = self.items.index(value)
-        i2 = i1 + 1
-        if i2 < len(self.items):
-            self.items[i1], self.items[i2] = self.items[i2], self.items[i1]
-            self.property.notify(self.object) # send a notification that this list has changed
-        else:
-            log.warning('Cannot move down last element')
+# vi:sw=4:et
