@@ -1,5 +1,3 @@
-#!/usr/bin/env python
-# vim:sw=4:et
 """
 Properties used to create the UML 2.0 data model.
 
@@ -32,9 +30,12 @@ from zope import component
 from collection import collection
 from event import AttributeChangeEvent, AssociationSetEvent, \
                   AssociationAddEvent, AssociationDeleteEvent
-from interfaces import IAttributeChangeEvent, IAssociationChangeEvent
+from event import DerivedUnionSetEvent, DerivedUnionAddEvent, \
+                  DerivedUnionDeleteEvent
+from event import RedefineSetEvent, RedefineAddEvent, RedefineDeleteEvent
+from interfaces import IAssociationChangeEvent, IAssociationSetEvent, \
+                       IAssociationAddEvent, IAssociationDeleteEvent
 import operator
-from gaphor.undomanager import get_undo_manager
 
 
 class umlproperty(object):
@@ -45,12 +46,6 @@ class umlproperty(object):
     connected, they will be notified when the value changes.
     """
 
-    def add_deriviate(self, deriviate):
-        try:
-            self.deriviates.append(deriviate)
-        except AttributeError:
-            self.deriviates = [ deriviate ]
-
     def __get__(self, obj, class_=None):
         if obj:
             return self._get(obj)
@@ -72,6 +67,12 @@ class umlproperty(object):
     def postload(self, obj):
         pass
 
+    def unlink(self, obj):
+        """
+        This is called from the Element to denote the element is unlinking.
+        """
+        pass
+
     def notify(self, obj):
         """
         Notify obj that the property's value has been changed.
@@ -82,42 +83,6 @@ class umlproperty(object):
         except Exception, e:
             log.error(str(e), e)
 
-        # we need to check if a deriviate is part of the current object:
-        # TODO: can we do this faster?
-        try:
-            deriviates = self.deriviates
-        except AttributeError:
-            pass # no derivedunion or redefine attribute
-        else:
-            values = []
-            for c in type(obj).__mro__:
-                values.extend(c.__dict__.values())
-
-            for d in deriviates:
-                if d in values:
-                    try:
-                        d.notify(obj)
-                    except Exception, e:
-                        log.error(e, e)
-
-
-#class undoattributeaction(undoaction):
-#    """
-#    This undo action contains one undo action for an attribute
-#    property.
-#    """
-#
-#    def undo(self):
-#        self.redo_value = self.prop._get(self.obj)
-#        setattr(self.obj, self.prop._name, self.value)
-#        self.prop.notify(self.obj)
-#        return self.redo
-#
-#    def redo(self):
-#        setattr(self.obj, self.prop._name, self.redo_value)
-#        self.prop.notify(self.obj)
-#        return self.undo
-
 
 class attribute(umlproperty):
     """
@@ -216,7 +181,6 @@ class enumeration(umlproperty):
         if value == old:
             return
 
-        #undoattributeaction(self, obj, self._get(obj))
         if value == self.default:
             delattr(obj, self._name)
         else:
@@ -227,7 +191,6 @@ class enumeration(umlproperty):
     def _del(self, obj, value=None):
         old = self._get(obj)
         try:
-            #undoattributeaction(self, obj, self._get(obj))
             delattr(obj, self._name)
         except AttributeError:
             pass
@@ -236,37 +199,6 @@ class enumeration(umlproperty):
             self.notify(obj)
 
 
-#class undosetassociationaction(undoaction):
-#    """
-#    Undo a 'set' action in an association.
-#    """
-#
-#    def undo(self):
-#        #log.debug('undosetassociationaction del: %s %s %s' % (self.obj, self.prop.name, self.value))
-#        self.prop._del(self.obj, self.value)
-#        return self.redo
-#
-#    def redo(self):
-#        #log.debug('undosetassociationaction set: %s %s %s' % (self.obj, self.prop.name, self.value))
-#        self.prop._set(self.obj, self.value)
-##        return self.undo
-
-
-#class undodelassociationaction(undoaction):
-#    """Undo a 'del' action in an association.
-#    """
-#
-#    def undo(self):
-#        #log.debug('undodelassociationaction set: %s %s %s' % (self.obj, self.prop.name, self.value))
-#        self.prop._set(self.obj, self.value)
-#        return self.redo
-#
-#    def redo(self):
-#        #log.debug('undodelassociationaction del: %s %s %s' % (self.obj, self.prop.name, self.value))
-#        self.prop._del(self.obj, self.value)
-#        return self.undo
-
-
 class association(umlproperty):
     """
     Association, both uni- and bi-directional.
@@ -277,9 +209,8 @@ class association(umlproperty):
     will cause the association to be ended if the element on the other end
     of the association is unlinked.
 
-    If the association is a composite relationship, the value is connected to
-    the elements __unlink__ signal too. This will cause the value to be
-    unlinked as soon as the element is unlinked.
+    If the association is a composite relationship, the association will
+    unlink all elements attached to if it is unlinked.
     """
  
     def __init__(self, name, type, lower=0, upper='*', composite=False, opposite=None):
@@ -297,7 +228,8 @@ class association(umlproperty):
         self._set(obj, value, do_notify=False)
 
     def postload(self, obj):
-        """In the postload step, ensure that bi-directional associations
+        """
+        In the postload step, ensure that bi-directional associations
         are bi-directional.
         """
         values = self._get(obj)
@@ -340,7 +272,7 @@ class association(umlproperty):
 
         This method is called from the opposite association property.
         """
-        #print '__set__', self, obj, value
+        #print '__set__', self, obj, value, self._get(obj)
         if not (isinstance(value, self.type) or \
                 (value is None and self.upper == 1)):
             raise AttributeError, 'Value should be of type %s' % self.type.__name__
@@ -349,25 +281,24 @@ class association(umlproperty):
             old = self._get(obj)
 
             # do nothing if we are assigned our current value:
+            # Still do your thing, since undo handlers expect that.
             if value is old:
                 return
 
-            # is done in _del(): undoassociationaction(self, obj, old)
             if old:
-                self._del(obj, old)
+                self._del(obj, old, from_opposite=from_opposite, do_notify=False)
+
+            if do_notify:
+                event = AssociationSetEvent(obj, self, old, value)
 
             if value is None:
+                if do_notify:
+                    self.notify(obj)
+                    component.handle(event)
                 return
 
-            #if value is self._get(obj):
-            #    #log.debug('association: value already in obj: %s' % value)
-            #    return
-
-            #if not from_opposite:
-            #    undosetassociationaction(self, obj, value)
             setattr(obj, self._name, value)
-            if do_notify:
-                component.handle(AssociationSetEvent(obj, self, old, value))
+
         else:
             # Set the actual value
             c = self._get(obj)
@@ -375,28 +306,26 @@ class association(umlproperty):
                 c = collection(self, obj, self.type)
                 setattr(obj, self._name, c)
             elif value in c:
-                #log.debug('association: value already in obj: %s' % value)
                 return
 
-            #if not from_opposite:
-            #    undosetassociationaction(self, obj, value)
             c.items.append(value)
             if do_notify:
-                component.handle(AssociationAddEvent(obj, self, value))
+                event = AssociationAddEvent(obj, self, value)
 
         # Callbacks are only connected if a new relationship has
         # been established.
         value.connect('__unlink__', self.__on_unlink, obj)
-        if self.composite:
-            obj.connect('__unlink__', self.__on_composite_unlink, value)
+#        if self.composite:
+#            obj.connect('__unlink__', self.__on_composite_unlink, value)
         
         if not from_opposite and self.opposite:
             getattr(type(value), self.opposite)._set(value, obj, from_opposite=True, do_notify=do_notify)
 
         if do_notify:
             self.notify(obj)
+            component.handle(event)
 
-    def _del(self, obj, value, from_opposite=False):
+    def _del(self, obj, value, from_opposite=False, do_notify=True):
         """
         Delete is used for element deletion and for removal of
         elements from a list.
@@ -410,12 +339,10 @@ class association(umlproperty):
             if value is None:
                 return
 
-        #if not from_opposite:
-        #    undodelassociationaction(self, obj, value)
-
         if not from_opposite and self.opposite:
             getattr(type(value), self.opposite)._del(value, obj, from_opposite=True)
 
+        event = None
         if self.upper > 1:
             c = self._get(obj)
             if c:
@@ -425,7 +352,9 @@ class association(umlproperty):
                 except:
                     pass
                 else:
-                    component.handle(AssociationDeleteEvent(obj, self, value))
+                    if do_notify:
+                        event = AssociationDeleteEvent(obj, self, value)
+
                 # Remove items collection if empty
                 if not items:
                     delattr(obj, self._name)
@@ -434,14 +363,29 @@ class association(umlproperty):
                 delattr(obj, self._name)
             except:
                 pass
-                #print 'association._del: delattr failed for %s' % self.name
             else:
-                component.handle(AssociationSetEvent(obj, self, value, None))
+                if do_notify:
+                    event = AssociationSetEvent(obj, self, value, None)
 
         value.disconnect(self.__on_unlink, obj)
-        if self.composite:
-            obj.disconnect(self.__on_composite_unlink, value)
-        self.notify(obj)
+#        if self.composite:
+#            obj.disconnect(self.__on_composite_unlink, value)
+        if do_notify and event:
+            self.notify(obj)
+            component.handle(event)
+
+    def unlink(self, obj):
+        values = self._get(obj)
+        composite = self.composite
+        if values:
+            if self.upper == 1:
+                values = [values]
+
+            for value in list(values):
+                # TODO: make normal unlinking work through this method.
+                #self.__delete__(obj, value)
+                if composite:
+                    value.unlink()
 
     def __on_unlink(self, value, pspec, obj):
         """
@@ -455,28 +399,14 @@ class association(umlproperty):
             # re-establish unlink handler:
             value.connect('__unlink__', self.__on_unlink, obj)
 
-    def __on_composite_unlink(self, obj, pspec, value):
-        """
-        Unlink value if we have a part-whole (composite) relationship
-        (value is a composite of obj).
-        The implementation of value.unlink() should ensure that no deadlocks
-        occur.
-        """
-        #print '__on_composite_unlink:', self, name, value
-        if pspec == '__unlink__':
-            value.unlink()
-            obj.connect('__unlink__', self.__on_composite_unlink, value)
-        #else:
-            #print 'RELINK'
-            #value.relink()
-
 
 class derivedunion(umlproperty):
-    """Derived union
-    Element.union = derivedunion('union', subset1, subset2..subsetn)
-    The subsets are the properties that participate in the union (Element.name),
+    """
+    Derived union
 
-    The derivedunion is added to the subsets deriviates list.
+      Element.union = derivedunion('union', subset1, subset2..subsetn)
+
+    The subsets are the properties that participate in the union (Element.name).
     """
 
     def __init__(self, name, lower, upper, *subsets):
@@ -484,59 +414,123 @@ class derivedunion(umlproperty):
         self._name = intern('_' + name)
         self.lower = lower
         self.upper = upper
-        self.subsets = subsets
+        self.subsets = set(subsets)
         self.single = len(subsets) == 1
-        for s in subsets:
-            s.add_deriviate(self)
+
+        component.provideHandler(self._association_changed)
+
 
     def load(self, obj, value):
         raise ValueError, 'Derivedunion: Properties should not be loaded in a derived union %s: %s' % (self.name, value)
 
+
     def save(self, obj, save_func):
         pass
 
+
     def __str__(self):
         return '<derivedunion %s: %s>' % (self.name, str(map(str, self.subsets))[1:-1])
 
-    def _get(self, obj):
+    def _union(self, obj, exclude=None):
+        """
+        Returns a union of all values as a set.
+        """
         if self.single:
-            #return getattr(obj, self.subsets[0])
-            return self.subsets[0].__get__(obj)
+            return iter(self.subsets).next().__get__(obj)
         else:
-            u = list()
+            u = set()
             for s in self.subsets:
+                if s is exclude:
+                    continue
                 tmp = s.__get__(obj)
                 if tmp:
-                    # append or extend tmp (is it a list or not)
                     try:
-                        for t in tmp:
-                            if t not in u:
-                                u.append(t)
-                        #u.extend(tmp)
+                        u.update(tmp)
                     except TypeError:
-                        if tmp not in u:
-                            u.append(tmp)
-            if self.upper > 1:
-                return u
-            else:
-                assert len(u) <= 1, 'Derived union %s of item %s should have length 1 %s' % (self.name, obj.id, tuple(u))
-                return u and u[0] or None
+                        # [0..1] property
+                        u.add(tmp)
+            return u
+
+    def _get(self, obj):
+        u = self._union(obj)
+        if self.upper > 1:
+            return u
+        else:
+            assert len(u) <= 1, 'Derived union %s of item %s should have length 1 %s' % (self.name, obj.id, tuple(u))
+            return u and iter(u).next() or None
+
 
     def _set(self, obj, value):
         raise AttributeError, 'Can not set values on a union'
 
+
     def _del(self, obj, value=None):
         raise AttributeError, 'Can not delete values on a union'
 
 
+    @component.adapter(IAssociationChangeEvent)
+    def _association_changed(self, event):
+        """
+        Re-emit state change for the derived union (as DerivedUnion*Event's).
+
+        TODO: We should fetch the old and new state of the namespace item in
+        stead of the old and new values of the item that changed.
+
+        If multiplicity is [0..1]:
+          send DerivedUnionSetEvent if len(union) < 2
+        if multiplicity is [*]:
+          send DerivedUnionAddEvent and DerivedUnionDeleteEvent
+            if value not in derived union and 
+        """
+        if event.property in self.subsets:
+            # mimic the events for Set/Add/Delete
+
+            if self.upper > 1:
+                if IAssociationSetEvent.providedBy(event):
+                    old_value, new_value = event.old_value, event.new_value
+                    if old_value and old_value not in self._union(event.element, exclude=event.property):
+                        component.handle(DerivedUnionDeleteEvent(event.element, self, old_value))
+                    if new_value and new_value not in self._union(event.element, exclude=event.property):
+                        component.handle(DerivedUnionAddEvent(event.element, self, new_value))
+
+                elif IAssociationAddEvent.providedBy(event):
+                    new_value = event.new_value
+                    if new_value not in self._union(event.element, exclude=event.property):
+                        component.handle(DerivedUnionAddEvent(event.element, self, new_value))
+                elif IAssociationDeleteEvent.providedBy(event):
+                    old_value = event.old_value
+                    if old_value not in self._union(event.element, exclude=event.property):
+                        component.handle(DerivedUnionDeleteEvent(event.element, self, old_value))
+                else:
+                    log.error('Don''t know how to handle event ' + str(event) + ' for derived union')
+            else:        
+                assert IAssociationSetEvent.providedBy(event)
+                old_value, new_value = event.old_value, event.new_value
+                # This is a [0..1] event
+                if self.single:
+                    # Only one subset element, so pass the values on
+                    component.handle(DerivedUnionSetEvent(event.element, self, old_value, new_value))
+                else:
+                    values = self._union(event.element, exclude=event.property)
+
+                    new_values = set(values)
+                    if new_value:
+                        new_values.add(new_value)
+                    if len(new_values) > 1:
+                        # In an in-between state. Do not emit notifications
+                        return
+                    if values:
+                        new_value = iter(values).next()
+                    component.handle(DerivedUnionSetEvent(event.element, self, old_value, new_value))
+            self.notify(event.element)
+
+
 class redefine(umlproperty):
     """
     Redefined association
     Element.x = redefine('x', Class, Element.assoc)
     If the redefine eclipses the original property (it has the same name)
     it ensures that the original values are saved and restored.
-
-    This property is connected to the originals deriviates list.
     """
 
     def __init__(self, name, type, original):
@@ -544,103 +538,71 @@ class redefine(umlproperty):
         self._name = intern('_' + name)
         self.type = type
         self.original = original
-        original.add_deriviate(self)
+
+        component.provideHandler(self._association_changed)
 
     upper = property(lambda s: s.original.upper)
     lower = property(lambda s: s.original.lower)
 
+
     def load(self, obj, value):
         if self.original.name == self.name:
             self.original.load(obj, value)
 
+
     def save(self, obj, save_func):
         if self.original.name == self.name:
             self.original.save(obj, save_func)
 
+
     def __str__(self):
         return '<redefine %s: %s = %s>' % (self.name, self.type.__name__, str(self.original))
 
+
     def __get__(self, obj, class_=None):
         # No longer needed
         if not obj:
             return self
         return self.original.__get__(obj, class_)
 
+
     def __set__(self, obj, value):
         # No longer needed
         if not isinstance(value, self.type):
             raise AttributeError, 'Value should be of type %s' % self.type.__name__
         self.original.__set__(obj, value)
 
+
     def __delete__(self, obj, value=None):
         # No longer needed
         self.original.__delete__(obj, value)
 
+
     def _get(self, obj):
         return self.original._get(obj)
 
+
     def _set(self, obj, value, from_opposite=False):
         return self.original._set(obj, value, from_opposite)
 
+
     def _del(self, obj, value, from_opposite=False):
         return self.original._del(obj, value, from_opposite)
 
 
-@component.adapter(IAttributeChangeEvent)
-def undo_attribute_change_event(event):
-    attribute = event.property
-    obj = event.element
-    value = event.old_value
-    def _undo_attribute_change_event():
-        attribute._set(obj, value)
-    get_undo_manager().add_undo_action(_undo_attribute_change_event)
-
-component.provideHandler(undo_attribute_change_event)
-
-
-@component.adapter(AssociationSetEvent)
-def undo_association_set_event(event):
-    association = event.property
-    obj = event.element
-    value = event.old_value
-    def _undo_association_set_event():
-        #print 'undoing action', obj, value
-        # Tell the assoctaion it should not need to let the opposite
-        # side connect (it has it's own signal)
-        association._set(obj, value, from_opposite=True)
-    get_undo_manager().add_undo_action(_undo_association_set_event)
-
-component.provideHandler(undo_association_set_event)
-
-
-@component.adapter(AssociationAddEvent)
-def undo_association_add_event(event):
-    association = event.property
-    obj = event.element
-    value = event.new_value
-    def _undo_association_add_event():
-        #print 'undoing action', obj, value
-        # Tell the assoctaion it should not need to let the opposite
-        # side connect (it has it's own signal)
-        association._del(obj, value, from_opposite=True)
-    get_undo_manager().add_undo_action(_undo_association_add_event)
-
-component.provideHandler(undo_association_add_event)
-
-
-@component.adapter(AssociationDeleteEvent)
-def undo_association_delete_event(event):
-    association = event.property
-    obj = event.element
-    value = event.old_value
-    def _undo_association_delete_event():
-        #print 'undoing action', obj, value
-        # Tell the assoctaion it should not need to let the opposite
-        # side connect (it has it's own signal)
-        association._set(obj, value, from_opposite=True)
-    get_undo_manager().add_undo_action(_undo_association_delete_event)
-
-component.provideHandler(undo_association_delete_event)
+    @component.adapter(IAssociationChangeEvent)
+    def _association_changed(self, event):
+        if event.property is self.original:
+            # mimic the events for Set/Add/Delete
+            if IAssociationSetEvent.providedBy(event):
+                component.handle(RedefineSetEvent(event.element, self, event.old_value, event.new_value))
+            elif IAssociationAddEvent.providedBy(event):
+                component.handle(RedefineAddEvent(event.element, self, event.new_value))
+            elif IAssociationDeleteEvent.providedBy(event):
+                component.handle(RedefineDeleteEvent(event.element, self, event.old_value))
+            else:
+                log.error('Don''t know how to handle event ' + str(event) + ' for redefined association')
+            self.notify(event.element)
 
 
 try:
@@ -654,3 +616,6 @@ else:
     psyco.bind(association)
     psyco.bind(derivedunion)
     psyco.bind(redefine)
+
+
+# vim:sw=4:et:ai
