@@ -4,32 +4,59 @@ Adapters
 
 from zope import interface, component
 
-from gaphas.item import NW, SE
+from gaphas.item import NW, NE, SW, SE
+from gaphas.canvas import CanvasProjection
+from gaphas.matrix import Matrix
 from gaphas import geometry
 from gaphas import constraint
 from gaphor import UML
+from gaphor.core import inject
 from gaphor.diagram.interfaces import IConnect
 from gaphor.diagram import items
 from gaphor.misc.ipair import ipair
 
 class AbstractConnect(object):
     """
-    Connection code, used by both ElementConnect and LineConnect.
+    Connection adapter for Gaphor diagram items.
+
+    Line item ``line`` connects with a handle to a connectable item ``element``.
+
+    Line constraint is created between ``line`` handle and two handles of ``element``
+    (called segment, see ``AbstractConnect._get_segment`` method).
+
+    Attributes:
+
+    - line: connecting item
+    - element: connectable item
+    - _canvas: canvas reference
+    - _matrix_e2l: element to line matrix
+    - _matrix_l2e: line to element matrix
     """
     interface.implements(IConnect)
 
     def __init__(self, element, line):
         self.element = element
         self.line = line
+        canvas = self._canvas = element.canvas
 
-    def glue(self, handle, x, y):
+        i2c = canvas.get_matrix_i2c
+        c2i = canvas.get_matrix_c2i
+        self._matrix_e2l = i2c(element) * c2i(line)
+        self._matrix_l2e = i2c(line) * c2i(element)
+
+        # code below returns None from transform_point... why?
+        # self._matrix_l2e = Matrix(*self._matrix_e2l)
+        # self._matrix_l2e.invert()
+
+
+    def glue(self, handle):
         """
         Return the point the handle could connect to. None if no connection
         is allowed.
         """
         raise NotImplemented, 'Implement glue() in the subclass'
 
-    def connect(self, handle, x, y):
+    def connect(self, handle):
         """
         Connect to an element. Note that at this point the line may
         be connected to some other, or the same element by means of the
@@ -42,17 +69,16 @@ class AbstractConnect(object):
         canvas = element.canvas
         solver = canvas.solver
 
-        pos = self.glue(handle, x, y)
-
         # Disconnect old model connection
         if handle.connected_to and handle.connected_to is not self.element:
             handle.disconnect()
  
-        # Stop here if no new connection should be established
-        if not pos:
+        # Save guard: only connect if it's permitted by glue()
+        #   -> glue() does not return None
+        if not self.glue(handle):
             return False
-        
-        self.connect_constraint(handle, pos[0], pos[1])
+
+        self.connect_constraints(handle)
 
         # Set disconnect handler in the adapter, so it will also wotk if
         # connections are created programmatically.
@@ -71,67 +97,103 @@ class AbstractConnect(object):
         self.disconnect_constraints(handle)
         handle.connected_to = None
 
-    def connect_constraint(self, handle, x, y):
+
+    def connect_constraints(self, handle):
         """
-        Create the actual constraint. The handle should be positioned before
-        this method is called.
-        x, y is the position the handle should have on the line.
+        Create the actual constraint. The handle should be moved into connection
+        position before this method is called.
         """
-        raise NotImplemented, 'Implement connect_constraint() in a subclass'
+        h1, h2 = self._get_segment(handle)
+
+        element = self.element
+        line = self.line
+
+        lc = constraint.LineConstraint(line=(CanvasProjection(h1.pos, element),
+                                             CanvasProjection(h2.pos, element)),
+                                       point=CanvasProjection(handle.pos, line))
+        handle._connect_constraint = lc
+        element.canvas.solver.add_constraint(lc)
+
+        handle.connected_to = element
+
 
     def disconnect_constraints(self, handle):
         """
         Disconnect() takes care of disconnecting the handle from the
         element it's attached to, by removing the constraints.
         """
-        solver = self.line.canvas.solver
         try:
-            solver.remove_constraint(handle._connect_constraint)
+            if handle._connect_constraint:
+                self.line.canvas.solver.remove_constraint(handle._connect_constraint)
         except AttributeError:
             pass # No _connect_constraint property yet
         handle._connect_constraint = None
 
 
+    def _get_segment(self, handle):
+        """
+        Get line segment, to which handle should connect to. Line segment is defined by two handles.
+        Examples of line segmets
+        - side of element item like class, action, etc.
+        - line segment of association
+        - lifeline's lifetime
+        """
+        raise NotImplemented()
+
+
+
 class ElementConnect(AbstractConnect):
     """
     Base class for connecting a line to an ElementItem class.
     """
-    
-    def side(self, handle_pos, glued):
+
+    def _get_segment(self, handle):
         """
         Determine the side on which the handle is connecting.
         This is done by determining the proximity to the nearest edge.
-        """
-        hx, hy = handle_pos
-        ax, ay = glued.handles()[NW].pos
-        bx, by = glued.handles()[SE].pos
-        return min((abs(hy - ay), 0), (abs(hx - bx), 1),
-                   (abs(hy - by), 2),  (abs(hx - ax), 3))[1]
 
-    def glue(self, handle, x, y):
+        Handles of one of the sides is returned.
+        """
+        handles = self.element.handles()
+        l2e = self._matrix_l2e
+        hx, hy = l2e.transform_point(*handle.pos)
+        ax, ay = handles[NW].x, handles[NW].y
+        bx, by = handles[SE].x, handles[SE].y
+
+        if abs(hx - ax) < 0.01:
+            return handles[NW], handles[SW]
+        elif abs(hy - ay) < 0.01:
+            return handles[NW], handles[NE]
+        elif abs(hx - bx) < 0.01:
+            return handles[NE], handles[SE]
+        else:
+            return handles[SW], handles[SE]
+        assert False
+
+
+    def bounds(self, element):
+        """
+        Returns bounds of the element that we're connecting to.
+        """
+        h = element.handles()
+        return map(float, (h[NW].pos + h[SE].pos))
+
+
+    def glue(self, handle):
         """
         Return the point the handle could connect to. None if no connection
         is allowed.
         """
-        h = self.element.handles()
-        bounds = map(float, (h[NW].pos + h[SE].pos))
-        return geometry.point_on_rectangle(bounds, (x, y), border=True)
+        l2e = self._matrix_l2e
+        e2l = self._matrix_e2l
 
-    def connect_constraint(self, handle, x, y):
-        """
-        Create the actual constraint. The handle should be positioned before
-        this method is called.
-        """
-        element = self.element
-        canvas = element.canvas
-        solver = canvas.solver
+        pos = l2e.transform_point(*handle.pos)
+        bounds = self.bounds(self.element)
+
+        pos = geometry.point_on_rectangle(bounds, pos, border=True)
+        pos = e2l.transform_point(*pos)
+        return pos
 
-        s = self.side((x, y), element)
-        handle._connect_constraint = \
-            constraint.LineConstraint(canvas, element, element.handles()[s],
-                                element.handles()[(s+1)%4], self.line, handle)
-        solver.add_constraint(handle._connect_constraint)
-        handle.connected_to = element
 
 
 class LineConnect(AbstractConnect):
@@ -145,56 +207,54 @@ class LineConnect(AbstractConnect):
     up relationships at model level too.
     """
 
-    def _glue(self, handle, x, y):
+    def _get_segment_data(self, handle):
         """
-        Return the segment and  point on the element (DiagramLine)
-        closest to (x, y).
+        Get segment data
+        - pos: position on a segment
+        - h1: handle starting segment
+        - h2: handle ending segment
         """
-        h = self.element.handles()
-        pos = (x, y)
-        min_d = None
-        segment = -1
-        min_p = None
         dlp = geometry.distance_line_point
-        for s, (h0, h1) in enumerate(ipair(h)):
-            d, p = dlp(h0.pos, h1.pos, pos)
-            if not s or d < min_d:
-                min_d = d
-                segment = s
-                min_p = p
-        return s, min_p
 
-    def glue(self, handle, x, y):
-        """
-        Return the point on the element (DiagramLine) closest to (x, y)
-        """
-        return self._glue(handle, x, y)[1]
+        handles = self.element.handles()
 
-    def connect_constraint(self, handle, x, y):
+        def segment(h1, h2, pos):
+            d, pos = dlp(h1.pos, h2.pos, pos)
+            return d, pos, h1, h2
+
+        pos = self._matrix_l2e.transform_point(*handle.pos)
+
+        # find the nearest segment from handle
+        data = (segment(h1, h2, pos) for h1, h2 in ipair(handles))
+        # No key needed, distance is first
+        d, pos, h1, h2 = min(data) #, key=lambda s: s[0])
+        return pos, h1, h2
+
+
+    def _get_segment(self, handle):
         """
-        Create the actual constraint. The handle should be positioned before
-        this method is called.
+        Return diagram line segment closest to a handle.
         """
-        element = self.element
-        canvas = element.canvas
-        solver = canvas.solver
+        _, h1, h2 = self._get_segment_data(handle)
+        return h1, h2
+
+
+    def glue(self, handle):
+        """
+        Return a point on a connectable element closest to a handle.
+        """
+        pos, _, _ = self._get_segment_data(handle)
+        return self._matrix_e2l.transform_point(*pos)
 
-        s, pos = self._glue(handle, x, y)
-        handle._connect_constraint = \
-            constraint.LineConstraint(canvas, element, element.handles()[s],
-                                element.handles()[s+1], self.line, handle)
-        solver.add_constraint(handle._connect_constraint)
-        handle.connected_to = element
 
 
 class CommentLineElementConnect(ElementConnect):
     """
-    Connect a comment line to a comment item.
-    Connect Comment.annotatedElement to any element
+    Connect a comment line to any element item.
     """
     component.adapts(items.ElementItem, items.CommentLineItem)
 
-    def glue(self, handle, x, y):
+    def glue(self, handle):
         """
         In addition to the normal check, both line ends may not be connected
         to the same element. Same goes for subjects.
@@ -214,15 +274,15 @@ class CommentLineElementConnect(ElementConnect):
             return None
 
         # One end should be connected to a CommentItem:
-        if connected_to and \
-                ((isinstance(connected_to, items.CommentItem) and isinstance(self.element, items.CommentItem)) or \
-                 (not isinstance(connected_to, items.CommentItem) and not isinstance(self.element, items.CommentItem))):
+        cls = items.CommentItem
+        glue_ok = isinstance(connected_to, cls) ^ isinstance(self.element, cls)
+        if connected_to and not glue_ok:
             return None
 
-        return super(CommentLineElementConnect, self).glue(handle, x, y)
+        return super(CommentLineElementConnect, self).glue(handle)
 
-    def connect(self, handle, x, y):
-        if super(CommentLineElementConnect, self).connect(handle, x, y):
+    def connect(self, handle):
+        if super(CommentLineElementConnect, self).connect(handle):
             opposite = self.line.opposite(handle)
             if opposite.connected_to:
                 if isinstance(opposite.connected_to.subject, UML.Comment):
@@ -244,12 +304,11 @@ component.provideAdapter(CommentLineElementConnect)
 
 class CommentLineLineConnect(LineConnect):
     """
-    Connect a comment line to a comment item.
-    Connect Comment.annotatedElement to any element
+    Connect a comment line to any diagram line.
     """
     component.adapts(items.DiagramLine, items.CommentLineItem)
 
-    def glue(self, handle, x, y):
+    def glue(self, handle):
         """
         In addition to the normal check, both line ends may not be connected
         to the same element. Same goes for subjects.
@@ -258,7 +317,9 @@ class CommentLineLineConnect(LineConnect):
         opposite = self.line.opposite(handle)
         element = self.element
         connected_to = opposite.connected_to
-        if connected_to is element:
+
+        # do not connect to the same item nor connect to other comment line
+        if connected_to is element or isinstance(element, items.CommentLineItem):
             return None
 
         # Same goes for subjects:
@@ -268,15 +329,15 @@ class CommentLineLineConnect(LineConnect):
             return None
 
         # One end should be connected to a CommentItem:
-        if connected_to and \
-                ((isinstance(connected_to, items.CommentItem) and isinstance(self.element, items.CommentItem)) or \
-                 (not isinstance(connected_to, items.CommentItem) and not isinstance(self.element, items.CommentItem))):
+        cls = items.CommentItem
+        glue_ok = isinstance(connected_to, cls) ^ isinstance(self.element, cls)
+        if connected_to and not glue_ok:
             return None
 
-        return super(CommentLineLineConnect, self).glue(handle, x, y)
+        return super(CommentLineLineConnect, self).glue(handle)
 
-    def connect(self, handle, x, y):
-        if super(CommentLineLineConnect, self).connect(handle, x, y):
+    def connect(self, handle):
+        if super(CommentLineLineConnect, self).connect(handle):
             opposite = self.line.opposite(handle)
             if opposite.connected_to and self.element.subject:
                 if isinstance(opposite.connected_to.subject, UML.Comment):
@@ -289,7 +350,7 @@ class CommentLineLineConnect(LineConnect):
         if handle.connected_to and opposite.connected_to:
             if isinstance(handle.connected_to.subject, UML.Comment):
                 del handle.connected_to.subject.annotatedElement[opposite.connected_to.subject]
-            else:
+            elif opposite.connected_to.subject:
                 del opposite.connected_to.subject.annotatedElement[handle.connected_to.subject]
         super(CommentLineLineConnect, self).disconnect(handle)
 
@@ -306,6 +367,8 @@ class RelationshipConnect(ElementConnect):
     on the canvas.
     """
 
+    element_factory = inject('element_factory')
+
     def relationship(self, required_type, head, tail):
         """
         Find an existing relationship in the model that meets the
@@ -366,14 +429,14 @@ class RelationshipConnect(ElementConnect):
         relation = self.relationship(type, head, tail)
         if not relation:
             line = self.line
-            relation = UML.create(type)
+            relation = self.element_factory.create(type)
             setattr(relation, head[0], line.head.connected_to.subject)
             setattr(relation, tail[0], line.tail.connected_to.subject)
         return relation
 
     def connect_connected_items(self, connected_items=None):
         """
-        Cause items connected to @line to reconnect, allowing them to
+        Cause items connected to ``line`` to reconnect, allowing them to
         establish or destroy relationships at model level.
         """
         line = self.line
@@ -385,7 +448,7 @@ class RelationshipConnect(ElementConnect):
         for item, handle in connected_items or line.canvas.get_connected_items(line):
             adapter = component.queryMultiAdapter((line, item), IConnect)
             assert adapter
-            adapter.connect(handle, handle.x, handle.y)
+            adapter.connect(handle)
         
     def disconnect_connected_items(self):
         """
@@ -426,12 +489,12 @@ class RelationshipConnect(ElementConnect):
         if old and len(old.presentation) == 0:
             old.unlink()
 
-    def connect(self, handle, x, y):
+    def connect(self, handle):
         """
         Connect the items to each other. The model level relationship
         is created by create_subject()
         """
-        if super(RelationshipConnect, self).connect(handle, x, y):
+        if super(RelationshipConnect, self).connect(handle):
             opposite = self.line.opposite(handle)
             if opposite.connected_to:
                 self.connect_subject(handle)
@@ -463,7 +526,7 @@ class DependencyConnect(RelationshipConnect):
     """
     component.adapts(items.NamedItem, items.DependencyItem)
 
-    def glue(self, handle, x, y):
+    def glue(self, handle):
         """
         In addition to the normal check, both line ends may not be connected
         to the same element. Same goes for subjects.
@@ -488,7 +551,7 @@ class DependencyConnect(RelationshipConnect):
            not isinstance(element.subject, UML.NamedElement):
             return None
 
-        return super(DependencyConnect, self).glue(handle, x, y)
+        return super(DependencyConnect, self).glue(handle)
 
     def connect_subject(self, handle):
         """
@@ -516,7 +579,7 @@ class ImplementationConnect(RelationshipConnect):
     """
     component.adapts(items.NamedItem, items.ImplementationItem)
 
-    def glue(self, handle, x, y):
+    def glue(self, handle):
         """
         In addition to the normal check, both line ends may not be connected
         to the same element. Same goes for subjects.
@@ -546,7 +609,7 @@ class ImplementationConnect(RelationshipConnect):
            not isinstance(element.subject, UML.BehavioredClassifier):
             return None
 
-        return super(ImplementationConnect, self).glue(handle, x, y)
+        return super(ImplementationConnect, self).glue(handle)
 
     def connect_subject(self, handle):
         relation = self.relationship_or_new(UML.Implementation,
@@ -564,7 +627,7 @@ class GeneralizationConnect(RelationshipConnect):
     component.adapts(items.ClassifierItem, items.GeneralizationItem)
 
 #    # FixMe: Both ends of the generalization should be of the same  type?
-    def glue(self, handle, x, y):
+    def glue(self, handle):
         """
         In addition to the normal check, both line ends may not be connected
         to the same element. Same goes for subjects.
@@ -584,7 +647,7 @@ class GeneralizationConnect(RelationshipConnect):
                  and connected_to.subject is element.subject:
             return None
 
-        return super(GeneralizationConnect, self).glue(handle, x, y)
+        return super(GeneralizationConnect, self).glue(handle)
 
     def connect_subject(self, handle):
         relation = self.relationship_or_new(UML.Generalization,
@@ -601,7 +664,7 @@ class IncludeConnect(RelationshipConnect):
     """
     component.adapts(items.UseCaseItem, items.IncludeItem)
 
-    def glue(self, handle, x, y):
+    def glue(self, handle):
         """
         In addition to the normal check, both line ends may not be connected
         to the same element. Same goes for subjects.
@@ -624,7 +687,7 @@ class IncludeConnect(RelationshipConnect):
         if not (element.subject and isinstance(element.subject, UML.UseCase)):
             return None
 
-        return super(IncludeConnect, self).glue(handle, x, y)
+        return super(IncludeConnect, self).glue(handle)
 
     def connect_subject(self, handle):
         relation = self.relationship_or_new(UML.Include,
@@ -641,7 +704,7 @@ class ExtendConnect(RelationshipConnect):
     """
     component.adapts(items.UseCaseItem, items.ExtendItem)
 
-    def glue(self, handle, x, y):
+    def glue(self, handle):
         """
         In addition to the normal check, both line ends may not be connected
         to the same element. Same goes for subjects.
@@ -664,13 +727,13 @@ class ExtendConnect(RelationshipConnect):
         if not (element.subject and isinstance(element.subject, UML.UseCase)):
             return None
 
-        return super(ExtendConnect, self).glue(handle, x, y)
+        return super(ExtendConnect, self).glue(handle)
 
     def connect_subject(self, handle):
         relation = self.relationship_or_new(UML.Extend,
                     ('extendedCase', None),
                     ('extension', 'extend'))
-        line.subject = relation
+        self.line.subject = relation
 
 component.provideAdapter(ExtendConnect)
 
@@ -681,7 +744,7 @@ class ExtensionConnect(RelationshipConnect):
     """
     component.adapts(items.ClassifierItem, items.ExtensionItem)
 
-    def glue(self, handle, x, y):
+    def glue(self, handle):
         """
         In addition to the normal check, both line ends may not be connected
         to the same element. Same goes for subjects.
@@ -711,7 +774,7 @@ class ExtensionConnect(RelationshipConnect):
            not isinstance(element.subject, UML.Stereotype):
             return None
 
-        return super(ExtensionConnect, self).glue(handle, x, y)
+        return super(ExtensionConnect, self).glue(handle)
 
     def connect_subject(self, handle):
         element = self.element
@@ -736,7 +799,7 @@ class ExtensionConnect(RelationshipConnect):
 
             # Find all associations and determine if the properties on
             # the association ends have a type that points to the class.
-            for assoc in UML.select():
+            for assoc in self.element_factory.select():
                 if isinstance(assoc, UML.Extension):
                     end1 = assoc.memberEnd[0]
                     end2 = assoc.memberEnd[1]
@@ -752,9 +815,9 @@ class ExtensionConnect(RelationshipConnect):
                             return
             else:
                 # Create a new Extension relationship
-                relation = UML.create(UML.Extension)
-                head_end = UML.create(UML.Property)
-                tail_end = UML.create(UML.ExtensionEnd)
+                relation = self.element_factory.create(UML.Extension)
+                head_end = self.element_factory.create(UML.Property)
+                tail_end = self.element_factory.create(UML.ExtensionEnd)
                 relation.package = element.canvas.diagram.namespace
                 relation.memberEnd = head_end
                 relation.memberEnd = tail_end
@@ -787,15 +850,11 @@ component.provideAdapter(ExtensionConnect)
 
 class AssociationConnect(RelationshipConnect):
     """
-    Connect Class and a Stereotype using an Extension
+    Connect association to classifier.
     """
     component.adapts(items.ClassifierItem, items.AssociationItem)
 
-    def glue(self, handle, x, y):
-        """
-        In addition to the normal check, both line ends may not be connected
-        to the same element. Same goes for subjects.
-        """
+    def glue(self, handle):
         opposite = self.line.opposite(handle)
         line = self.line
         element = self.element
@@ -805,7 +864,7 @@ class AssociationConnect(RelationshipConnect):
         if not isinstance(element.subject, UML.Classifier):
             return None
 
-        return super(AssociationConnect, self).glue(handle, x, y)
+        return super(AssociationConnect, self).glue(handle)
 
     def connect_subject(self, handle):
         element = self.element
@@ -827,7 +886,7 @@ class AssociationConnect(RelationshipConnect):
                     
             # Find all associations and determine if the properties on
             # the association ends have a type that points to the class.
-            for assoc in UML.select():
+            for assoc in self.element_factory.select():
                 if isinstance(assoc, UML.Association):
                     end1 = assoc.memberEnd[0]
                     end2 = assoc.memberEnd[1]
@@ -849,11 +908,11 @@ class AssociationConnect(RelationshipConnect):
                             return
             else:
                 # Create a new Extension relationship
-                relation = UML.create(UML.Association)
-                head_end = UML.create(UML.Property)
-                head_end.lowerValue = UML.create(UML.LiteralSpecification)
-                tail_end = UML.create(UML.Property)
-                tail_end.lowerValue = UML.create(UML.LiteralSpecification)
+                relation = self.element_factory.create(UML.Association)
+                head_end = self.element_factory.create(UML.Property)
+                head_end.lowerValue = self.element_factory.create(UML.LiteralSpecification)
+                tail_end = self.element_factory.create(UML.Property)
+                tail_end.lowerValue = self.element_factory.create(UML.LiteralSpecification)
                 relation.package = element.canvas.diagram.namespace
                 relation.memberEnd = head_end
                 relation.memberEnd = tail_end
@@ -862,9 +921,10 @@ class AssociationConnect(RelationshipConnect):
                 head_type.ownedAttribute = tail_end
                 tail_type.ownedAttribute = head_end
 
-                line.subject = relation
                 line.head_end.subject = head_end
                 line.tail_end.subject = tail_end
+                # Do subject itself last, so event handlers can trigger
+                line.subject = relation
 
     def disconnect_subject(self, handle):
         """
@@ -892,7 +952,7 @@ class FlowConnect(RelationshipConnect):
     Connect FlowItem and Action/ObjectNode, initial/final nodes.
     """
 
-    def glue(self, handle, x, y):
+    def glue(self, handle):
         """
         In addition to the normal check, both line ends may not be connected
         to the same element. Same goes for subjects.
@@ -907,7 +967,7 @@ class FlowConnect(RelationshipConnect):
            or handle is line.tail and isinstance(subject, UML.InitialNode):
             return None
 
-        return super(FlowConnect, self).glue(handle, x, y)
+        return super(FlowConnect, self).glue(handle)
 
     def connect_subject(self, handle):
         line = self.line
@@ -924,7 +984,7 @@ class FlowConnect(RelationshipConnect):
                         ('source', 'outgoing'),
                         ('target', 'incoming'))
         if not relation.guard:
-            relation.guard = UML.create(UML.LiteralSpecification)
+            relation.guard = self.element_factory.create(UML.LiteralSpecification)
         line.subject = relation
         opposite = line.opposite(handle)
         if opposite and isinstance(opposite.connected_to, (items.ForkNodeItem, items.DecisionNodeItem)):
@@ -952,8 +1012,7 @@ class FlowForkDecisionNodeConnect(FlowConnect):
     Abstract class with common behaviour for Fork/Join node and
     Decision/Merge node.
     """
-
-    def glue(self, handle, x, y):
+    def glue(self, handle):
         """
         In addition to the normal check, one end should have at most one
         edge (incoming or outgoing).
@@ -985,7 +1044,11 @@ class FlowForkDecisionNodeConnect(FlowConnect):
         #if handle is line.tail and len(subject.incoming) > 0 and len(subject.outgoing) > 1:
         #    return None
 
-        return super(FlowForkDecisionNodeConnect, self).glue(handle, x, y)
+#        assert len(self.element.handles()) == 2, '%s: %d' % (self.element, len(self.element.handles()))
+#        h1, h2 = self.element.handles()
+#        return geometry.distance_line_point(h1.pos, h2.pos, (x, y))[1]
+
+        return super(FlowForkDecisionNodeConnect, self).glue(handle)
 
     def combine_nodes(self):
         """
@@ -997,10 +1060,10 @@ class FlowForkDecisionNodeConnect(FlowConnect):
         element = self.element
         subject = element.subject
         if len(subject.incoming) > 1 and len(subject.outgoing) < 2:
-            UML.swap_element(subject, join_node_class)
+            self.element_factory.swap_element(subject, join_node_class)
             element.request_update()
         elif len(subject.incoming) < 2 and len(subject.outgoing) > 1:
-            UML.swap_element(subject, fork_node_class)
+            self.element_factory.swap_element(subject, fork_node_class)
             element.request_update()
         elif not element.combined and len(subject.incoming) > 1 and len(subject.outgoing) > 1:
             join_node = subject
@@ -1011,11 +1074,11 @@ class FlowForkDecisionNodeConnect(FlowConnect):
             else:
                 flow_class = UML.ControlFlow
             
-            UML.swap_element(join_node, join_node_class)
-            fork_node = UML.create(fork_node_class)
+            self.element_factory.swap_element(join_node, join_node_class)
+            fork_node = self.element_factory.create(fork_node_class)
             for flow in list(join_node.outgoing):
                 flow.source = fork_node
-            flow = UML.create(flow_class)
+            flow = self.element_factory.create(flow_class)
             flow.source = join_node
             flow.target = fork_node
 
@@ -1047,7 +1110,7 @@ class FlowForkDecisionNodeConnect(FlowConnect):
                 # swap subject to fork node if outgoing > 1
                 if len(join_node.outgoing) > 1:
                     assert len(join_node.incoming) < 2
-                    UML.swap_element(join_node, fork_node_class)
+                    self.element_factory.swap_element(join_node, fork_node_class)
                 element.combined = None
 
     def connect_subject(self, handle):
@@ -1063,21 +1126,28 @@ class FlowForkDecisionNodeConnect(FlowConnect):
         # of incoming/outgoing edges.
         self.combine_nodes()
 
-    def disconnect_subject(self):
-        super(FlowForkDecisionNodeConnect, self).disconnect_subject()
+    def disconnect_subject(self, handle):
+        super(FlowForkDecisionNodeConnect, self).disconnect_subject(handle)
         if self.element.combined:
             self.decombine_nodes()
 
 
 class FlowForkNodeConnect(FlowForkDecisionNodeConnect):
     """
-    Connect Flow to a ForkNode
+    Connect Flow to a ForkNode.
     """
     component.adapts(items.ForkNodeItem, items.FlowItem)
 
     fork_node_class=UML.ForkNode
     join_node_class=UML.JoinNode
 
+    def _get_segment(self, handle):
+        return self.element.handles()
+
+    def bounds(self, element):
+        h1, h2 = element.handles()
+        return map(float, (h1.pos + h2.pos))
+
 component.provideAdapter(FlowForkNodeConnect)
 
 
@@ -1093,4 +1163,178 @@ class FlowDecisionNodeConnect(FlowForkDecisionNodeConnect):
 component.provideAdapter(FlowDecisionNodeConnect)
 
 
+class MessageLifelineConnect(ElementConnect):
+    """
+    Connect lifeline with a message.
+
+    A message can connect to both the lifeline's head (the rectangle)
+    or the lifetime line. In case it's added to the head, the message
+    is considered to be part of a communication diagram. If the message is
+    added to a lifetime line, it's considered a sequence diagram.
+
+    """
+
+    component.adapts(items.LifelineItem, items.MessageItem)
+
+    element_factory = inject('element_factory')
+
+    def connect_lifelines(self, line, send, received):
+        """
+        Always create a new Message with two EventOccurence instances.
+        """
+        def get_subject():
+            if not line.subject:
+                message = self.element_factory.create(UML.Message)
+                message.name = 'call()'
+                line.subject = message
+            return line.subject
+
+        if send:
+            message = get_subject()
+            if not message.sendEvent:
+                event = self.element_factory.create(UML.EventOccurrence)
+                event.sendMessage = message
+                event.covered = send.subject
+
+        if received:
+            message = get_subject()
+            if not message.receiveEvent:
+                event = self.element_factory.create(UML.EventOccurrence)
+                event.receiveMessage = message
+                event.covered = received.subject
+
+
+    def disconnect_lifelines(self, line):
+        """
+        Disconnect lifeline and set appropriate kind of message item. If
+        there are no lifelines connected on both ends, then remove the message
+        from the data model.
+        """
+        send = line.head.connected_to
+        received = line.tail.connected_to
+
+        if send:
+            event = line.subject.receiveEvent
+            if event:
+                event.unlink()
+
+        if received:
+            event = line.subject.sendEvent
+            if event:
+                event.unlink()
+
+        if not send and not received:
+            # Both ends are disconnected:
+            message = line.subject
+            del line.subject
+            if not message.presentation:
+                message.unlink()
+            for message in list(line._messages):
+                line.remove_message(message, False)
+                message.unlink()
+
+            for message in list(line._inverted_messages):
+                line.remove_message(message, True)
+                message.unlink()
+
+
+    def _glue_lifetime(self, handle):
+        """
+        Glue ``handle`` to lifeline's lifetime.
+
+        The position returned is in line coordinates.
+        """
+        lifetime = self.element.lifetime
+        top = lifetime.top.pos
+        bottom = lifetime.bottom.pos
+
+        pos = self._matrix_l2e.transform_point(*handle.pos)
+        d, pos = geometry.distance_line_point(top, bottom, pos)
+
+        return self._matrix_e2l.transform_point(*pos)
+
+
+    def glue(self, handle):
+        """
+        Glue to lifeline's head or lifetime. If lifeline's lifetime is
+        visible then disallow connection to lifeline's head.
+        """
+        element = self.element
+        lifetime = element.lifetime
+        line = self.line
+        opposite = line.opposite(handle)
+
+        pos = None
+        if opposite.connected_to:
+            opposite_is_visible = opposite.connected_to.lifetime.is_visible
+
+            if lifetime.is_visible and opposite_is_visible:
+                # glue lifetimes if both are visible
+                pos = self._glue_lifetime(handle)
+            elif not lifetime.is_visible and not opposite_is_visible:
+                # glue heads if both lifetimes are invisible
+                pos = ElementConnect.glue(self, handle)
+
+        elif lifetime.is_visible:
+            pos = self._glue_lifetime(handle)
+
+        else:
+            pos = ElementConnect.glue(self, handle)
+        return pos
+        
+
+    def _get_segment(self, handle):
+        """
+        Return handles of one of lifeline head's side or lifetime handles.
+        """
+        element = self.element
+        if element.lifetime.is_visible:
+            return element.handles()[-2:] # return lifeline's lifetime handles
+        else:
+            return super(MessageLifelineConnect, self)._get_segment(handle)
+        assert False
+
+
+    def connect(self, handle):
+        if not ElementConnect.connect(self, handle):
+            return
+
+        line = self.line
+        send = line.head.connected_to
+        received = line.tail.connected_to
+        self.connect_lifelines(line, send, received)
+
+        lifetime = self.element.lifetime
+        # if no lifetime then disallow making lifetime visible
+        if not lifetime.is_visible:
+            lifetime.bottom.movable = False
+        # todo: move code above to LifetimeItem class
+        lifetime._messages_count += 1
+
+
+    def disconnect(self, handle):
+        line = self.line
+        send = line.head.connected_to
+        received = line.tail.connected_to
+
+        # if a message is delete message, then disconnection causes
+        # lifeline to be no longer destroyed (note that there can be
+        # only one delete message connected to lifeline)
+        if received and line.subject.messageSort == 'deleteMessage':
+            received.lifetime.is_destroyed = False
+            received.request_update()
+
+        ElementConnect.disconnect(self, handle)
+        self.disconnect_lifelines(line)
+
+        lifetime = self.element.lifetime
+        lifetime._messages_count -= 1
+        # if there are no messages connected then allow to make lifetime visible
+        # todo: move code below to LifetimeItem class
+        if lifetime._messages_count < 1:
+            lifetime.bottom.movable = True
+
+
+component.provideAdapter(MessageLifelineConnect)
+
 # vim:sw=4:et:ai
