@@ -2,38 +2,31 @@
 Test Item connections.
 """
 
-import unittest
+from gaphor.tests import TestCase
 from zope import component
-from gaphor import resource
 from gaphor import UML
-from gaphor.ui.mainwindow import MainWindow
 from gaphor.diagram import items
 from gaphor.diagram.interfaces import IConnect
 
-# Ensure adapters are loaded
-import gaphor.adapters
+class ConnectorTestCase(TestCase):
 
-class ConnectorTestCase(unittest.TestCase):
-
-    def tearDown(self):
-        UML.flush()
+    services = ['element_factory', 'adapter_loader']
 
     def test_commentline_element(self):
         """
 	Test CommentLineItem connecting to comment and Actor items.
         """
-        diagram = UML.create(UML.Diagram)
-        comment = diagram.create(items.CommentItem, subject=UML.create(UML.Comment))
-        line = diagram.create(items.CommentLineItem)
-        actor = diagram.create(items.ActorItem, subject=UML.create(UML.Actor))
-        actor2 = diagram.create(items.ActorItem, subject=UML.create(UML.Actor))
+        comment = self.create(items.CommentItem, UML.Comment)
+        line = self.create(items.CommentLineItem)
+        actor = self.create(items.ActorItem, UML.Actor)
+        actor2 = self.create(items.ActorItem, UML.Actor)
 
         # Connect the comment item to the head of the line:
 
         adapter = component.queryMultiAdapter((comment, line), IConnect)
 
         handle = line.head
-        adapter.connect(handle, handle.x, handle.y)
+        adapter.connect(handle)
 
         assert handle.connected_to is comment
         assert handle._connect_constraint is not None
@@ -42,9 +35,9 @@ class ConnectorTestCase(unittest.TestCase):
         # Connecting two ends of the line to the same item is not allowed:
 
         handle = line.tail
-        adapter.connect(handle, handle.x, handle.y)
+        adapter.connect(handle)
 
-        assert handle.connected_to is None
+        assert handle.connected_to is None, handle.connected_to
         assert not hasattr(handle,'_connect_constraint')
         assert not comment.subject.annotatedElement, comment.subject.annotatedElement
 
@@ -53,7 +46,7 @@ class ConnectorTestCase(unittest.TestCase):
         adapter = component.queryMultiAdapter((actor, line), IConnect)
 
         handle = line.handles()[-1]
-        adapter.connect(handle, handle.x, handle.y)
+        adapter.connect(handle)
 
         assert handle.connected_to is actor
         assert handle._connect_constraint is not None
@@ -65,7 +58,7 @@ class ConnectorTestCase(unittest.TestCase):
 
         handle = line.tail
         adapter = component.queryMultiAdapter((actor2, line), IConnect)
-        adapter.connect(handle, handle.x, handle.y)
+        adapter.connect(handle)
 
         assert handle.connected_to is actor2
         assert handle._connect_constraint is not None
@@ -93,23 +86,21 @@ class ConnectorTestCase(unittest.TestCase):
           2. association is disconnected while a comment is connected:
              -> association should be removed from comment.annotatedElement
         """
-        diagram = UML.create(UML.Diagram)
-        comment = diagram.create(items.CommentItem, subject=UML.create(UML.Comment))
-        line = diagram.create(items.CommentLineItem)
+        comment = self.create(items.CommentItem, UML.Comment)
+        line = self.create(items.CommentLineItem)
         line.head.pos = 100, 100
         line.tail.pos = 100, 100
-        c1 = diagram.create(items.ClassItem, subject=UML.create(UML.Class))
-        c2 = diagram.create(items.ClassItem, subject=UML.create(UML.Class))
-        assoc = diagram.create(items.AssociationItem)
+        c1 = self.create(items.ClassItem, UML.Class)
+        c2 = self.create(items.ClassItem, UML.Class)
+        assoc = self.create(items.AssociationItem)
 
         adapter = component.queryMultiAdapter((c1, assoc), IConnect)
         handle = assoc.head
-        adapter.connect(handle, handle.x, handle.y)
+        adapter.connect(handle)
 
         adapter = component.queryMultiAdapter((c2, assoc), IConnect)
         handle = assoc.tail
-        adapter.connect(handle, handle.x, handle.y)
-
+        adapter.connect(handle) 
         assert assoc.head.connected_to is c1
         assert assoc.tail.connected_to is c2
         assert assoc.subject
@@ -118,11 +109,12 @@ class ConnectorTestCase(unittest.TestCase):
 
         adapter = component.queryMultiAdapter((assoc, line), IConnect)
         assert adapter
+        import gaphor.adapters.connectors
         assert type(adapter) is gaphor.adapters.connectors.CommentLineLineConnect
         handle = line.head
-        pos = adapter.glue(handle, handle.x, handle.y)
-        assert pos == (0, 10), pos
-        adapter.connect(handle, handle.x, handle.y)
+        pos = adapter.glue(handle)
+        assert pos == (10, 50), pos
+        adapter.connect(handle)
 
         assert handle.connected_to is assoc
         assert handle._connect_constraint is not None
@@ -131,7 +123,7 @@ class ConnectorTestCase(unittest.TestCase):
         # Connecting two ends of the line to the same item is not allowed:
 
         handle = line.tail
-        adapter.connect(handle, handle.x, handle.y)
+        adapter.connect(handle)
 
         assert handle.connected_to is None
         assert not hasattr(handle,'_connect_constraint')
@@ -142,7 +134,7 @@ class ConnectorTestCase(unittest.TestCase):
         adapter = component.queryMultiAdapter((comment, line), IConnect)
 
         handle = line.tail
-        adapter.connect(handle, handle.x, handle.y)
+        adapter.connect(handle)
 
         assert handle.connected_to is comment
         assert handle._connect_constraint is not None
@@ -160,7 +152,7 @@ class ConnectorTestCase(unittest.TestCase):
 
         # Connect again:
 
-        adapter.connect(handle, handle.x, handle.y)
+        adapter.connect(handle)
         assert handle.connected_to is not None, handle.connected_to
 
 
@@ -169,21 +161,20 @@ class ConnectorTestCase(unittest.TestCase):
         Test behaviour when the CommentLine's subject (association) is
         connected after the comment line is connected.
         """
-        diagram = UML.create(UML.Diagram)
-        comment = diagram.create(items.CommentItem, subject=UML.create(UML.Comment))
-        line = diagram.create(items.CommentLineItem)
+        comment = self.create(items.CommentItem, UML.Comment)
+        line = self.create(items.CommentLineItem)
         line.head.pos = 100, 100
         line.tail.pos = 100, 100
-        c1 = diagram.create(items.ClassItem, subject=UML.create(UML.Class))
-        c2 = diagram.create(items.ClassItem, subject=UML.create(UML.Class))
-        assoc = diagram.create(items.AssociationItem)
+        c1 = self.create(items.ClassItem, UML.Class)
+        c2 = self.create(items.ClassItem, UML.Class)
+        assoc = self.create(items.AssociationItem)
 
         # connect the comment
 
         adapter = component.queryMultiAdapter((comment, line), IConnect)
 
         handle = line.tail
-        adapter.connect(handle, handle.x, handle.y)
+        adapter.connect(handle)
 
         assert handle.connected_to is comment
         assert handle._connect_constraint is not None
@@ -193,7 +184,7 @@ class ConnectorTestCase(unittest.TestCase):
 
         adapter = component.queryMultiAdapter((assoc, line), IConnect)
         handle = line.head
-        adapter.connect(handle, handle.x, handle.y)
+        adapter.connect(handle)
 
         assert handle.connected_to is assoc
         assert handle._connect_constraint is not None
@@ -204,7 +195,7 @@ class ConnectorTestCase(unittest.TestCase):
 
         adapter = component.queryMultiAdapter((c1, assoc), IConnect)
         handle = assoc.head
-        adapter.connect(handle, handle.x, handle.y)
+        adapter.connect(handle)
 
         assert handle.connected_to is c1
         assert handle._connect_constraint is not None
@@ -213,7 +204,7 @@ class ConnectorTestCase(unittest.TestCase):
 
         adapter = component.queryMultiAdapter((c2, assoc), IConnect)
         handle = assoc.tail
-        adapter.connect(handle, handle.x, handle.y)
+        adapter.connect(handle)
 
         assert assoc.head.connected_to is c1
         assert assoc.tail.connected_to is c2
@@ -236,25 +227,24 @@ class ConnectorTestCase(unittest.TestCase):
 
 
     def test_dependency(self):
-        diagram = UML.create(UML.Diagram)
-        actor1 = diagram.create(items.ActorItem, subject=UML.create(UML.Actor))
-        actor2 = diagram.create(items.ActorItem, subject=UML.create(UML.Actor))
-        dep = diagram.create(items.DependencyItem)
+        actor1 = self.create(items.ActorItem, UML.Actor)
+        actor2 = self.create(items.ActorItem, UML.Actor)
+        dep = self.create(items.DependencyItem)
 
         adapter = component.queryMultiAdapter((actor1, dep), IConnect)
 
-        adapter.connect(dep.head, dep.head.x, dep.head.y)
+        adapter.connect(dep.head)
 
         assert dep.subject is None
         assert dep.head.connected_to is actor1
 
         adapter = component.queryMultiAdapter((actor2, dep), IConnect)
 
-        adapter.connect(dep.tail, dep.tail.x, dep.tail.y)
+        adapter.connect(dep.tail)
 
         assert dep.subject is not None
         assert isinstance(dep.subject, UML.Dependency), dep.subject
-        assert dep.subject in UML.select()
+        assert dep.subject in self.element_factory.select(), self.element_factory.lselect()
         assert dep.head.connected_to is actor1
         assert dep.tail.connected_to is actor2
 
@@ -267,13 +257,13 @@ class ConnectorTestCase(unittest.TestCase):
 
         assert dep.subject is None
         assert dep.tail.connected_to is None
-        assert dep_subj not in UML.select()
+        assert dep_subj not in self.element_factory.select()
         assert dep_subj not in actor1.subject.supplierDependency
         assert dep_subj not in actor2.subject.clientDependency
 
-        #iface1 = diagram.create(items.InterfaceItem, subject=UML.Interface)
+        #iface1 = self.create(items.InterfaceItem, UML.Interface)
 
-        adapter.connect(dep.tail, dep.tail.x, dep.tail.y)
+        adapter.connect(dep.tail)
 
         assert dep.subject is not None
         assert dep.subject is not dep_subj # the old subject has been deleted
@@ -288,20 +278,19 @@ class ConnectorTestCase(unittest.TestCase):
         """Dependency should appear in a new diagram, bound on a new
         DependencyItem.
         """
-        diagram = UML.create(UML.Diagram)
-        actor1 = UML.create(UML.Actor)
-        actor2 = UML.create(UML.Actor)
-        actoritem1 = diagram.create(items.ActorItem, subject=actor1)
-        actoritem2 = diagram.create(items.ActorItem, subject=actor2)
-        dep = diagram.create(items.DependencyItem)
+        actoritem1 = self.create(items.ActorItem, UML.Actor)
+        actoritem2 = self.create(items.ActorItem, UML.Actor)
+        actor1 = actoritem1.subject
+        actor2 = actoritem2.subject
+        dep = self.create(items.DependencyItem)
         
         adapter = component.queryMultiAdapter((actoritem1, dep), IConnect)
 
-        adapter.connect(dep.head, dep.head.x, dep.head.y)
+        adapter.connect(dep.head)
 
         adapter = component.queryMultiAdapter((actoritem2, dep), IConnect)
 
-        adapter.connect(dep.tail, dep.tail.x, dep.tail.y)
+        adapter.connect(dep.tail)
 
         assert dep.subject
         assert len(actor1.supplierDependency) == 1
@@ -311,18 +300,18 @@ class ConnectorTestCase(unittest.TestCase):
 
         # Do the same thing, but now on a new diagram:
 
-        diagram2 = UML.create(UML.Diagram)
+        diagram2 = self.element_factory.create(UML.Diagram)
         actoritem3 = diagram2.create(items.ActorItem, subject=actor1)
         actoritem4 = diagram2.create(items.ActorItem, subject=actor2)
         dep2 = diagram2.create(items.DependencyItem)
 
         adapter = component.queryMultiAdapter((actoritem3, dep2), IConnect)
 
-        adapter.connect(dep2.head, dep2.head.x, dep2.head.y)
+        adapter.connect(dep2.head)
 
         adapter = component.queryMultiAdapter((actoritem4, dep2), IConnect)
 
-        adapter.connect(dep2.tail, dep2.tail.x, dep2.tail.y)
+        adapter.connect(dep2.tail)
 
         assert dep2.subject
         assert len(actor1.supplierDependency) == 1
@@ -333,48 +322,46 @@ class ConnectorTestCase(unittest.TestCase):
         assert dep.subject is dep2.subject
 
     def test_implementation(self):
-        diagram = UML.create(UML.Diagram)
-        impl = diagram.create(items.ImplementationItem)
-        clazz = diagram.create(items.ClassItem, subject=UML.create(UML.Class))
-        iface = diagram.create(items.InterfaceItem, subject=UML.create(UML.Interface))
+        impl = self.create(items.ImplementationItem)
+        clazz = self.create(items.ClassItem, UML.Class)
+        iface = self.create(items.InterfaceItem, UML.Interface)
 
         adapter = component.queryMultiAdapter((clazz, impl), IConnect)
 
-        adapter.connect(impl.head, impl.head.x, impl.head.y)
+        adapter.connect(impl.head)
 
         # Should not be allowed to connect to anything but Interfaces
 
         assert impl.head.connected_to is None
 
-        adapter.connect(impl.tail, impl.tail.x, impl.tail.y)
+        adapter.connect(impl.tail)
         assert impl.tail.connected_to is clazz
         assert impl.subject is None
 
         adapter = component.queryMultiAdapter((iface, impl), IConnect)
 
-        adapter.connect(impl.head, impl.head.x, impl.head.y)
-
+        adapter.connect(impl.head)
+        
         assert impl.head.connected_to is iface
         assert impl.subject is not None
         assert impl.subject.contract[0] is iface.subject
         assert impl.subject.implementatingClassifier[0] is clazz.subject
 
     def test_generalization(self):
-        diagram = UML.create(UML.Diagram)
-        gen = diagram.create(items.GeneralizationItem)
-        c1 = diagram.create(items.ClassItem, subject=UML.create(UML.Class))
-        c2 = diagram.create(items.ClassItem, subject=UML.create(UML.Class))
+        gen = self.create(items.GeneralizationItem)
+        c1 = self.create(items.ClassItem, UML.Class)
+        c2 = self.create(items.ClassItem, UML.Class)
 
         adapter = component.queryMultiAdapter((c1, gen), IConnect)
 
-        adapter.connect(gen.tail, gen.tail.x, gen.tail.y)
+        adapter.connect(gen.tail)
 
         assert gen.tail.connected_to is c1
         assert gen.subject is None
 
         adapter = component.queryMultiAdapter((c2, gen), IConnect)
 
-        adapter.connect(gen.head, gen.head.x, gen.head.y)
+        adapter.connect(gen.head)
 
         assert gen.head.connected_to is c2
         assert gen.subject is not None
@@ -382,61 +369,55 @@ class ConnectorTestCase(unittest.TestCase):
         assert gen.subject.specific is c1.subject
 
     def test_extension(self):
-        assert len(list(UML.select())) == 0
+        gen = self.create(items.ExtensionItem)
+        c1 = self.create(items.ClassItem, UML.Stereotype)
+        c2 = self.create(items.ClassItem, UML.Class)
 
-        diagram = UML.create(UML.Diagram)
-        gen = diagram.create(items.ExtensionItem)
-        c1 = diagram.create(items.ClassItem, subject=UML.create(UML.Stereotype))
-        c2 = diagram.create(items.ClassItem, subject=UML.create(UML.Class))
-
-        assert len(list(UML.select())) == 3
+        assert len(list(self.element_factory.select())) == 3
 
         adapter = component.queryMultiAdapter((c1, gen), IConnect)
 
-        adapter.connect(gen.tail, gen.tail.x, gen.tail.y)
+        adapter.connect(gen.tail)
 
         assert gen.tail.connected_to is c1
         assert gen.subject is None
 
         adapter = component.queryMultiAdapter((c2, gen), IConnect)
 
-        adapter.connect(gen.head, gen.head.x, gen.head.y)
+        adapter.connect(gen.head)
 
         assert gen.head.connected_to is c2
         assert gen.subject is not None
         
-        assert len(list(UML.select())) == 6
+        assert len(list(self.element_factory.select())) == 6, len(list(self.element_factory.select()))
 
         adapter.disconnect(gen.head)
 
-        assert len(list(UML.select())) == 3, list(UML.select())
+        assert len(list(self.element_factory.select())) == 3, list(self.element_factory.select())
 
     def test_association(self):
-        assert len(list(UML.select())) == 0
+        gen = self.create(items.AssociationItem)
+        c1 = self.create(items.ClassItem, UML.Class)
+        c2 = self.create(items.ClassItem, UML.Class)
 
-        diagram = UML.create(UML.Diagram)
-        gen = diagram.create(items.AssociationItem)
-        c1 = diagram.create(items.ClassItem, subject=UML.create(UML.Class))
-        c2 = diagram.create(items.ClassItem, subject=UML.create(UML.Class))
-
-        assert len(list(UML.select())) == 3
+        assert len(list(self.element_factory.select())) == 3
 
         adapter = component.queryMultiAdapter((c1, gen), IConnect)
 
-        adapter.connect(gen.tail, gen.tail.x, gen.tail.y)
+        adapter.connect(gen.tail)
 
         assert gen.tail.connected_to is c1
         assert gen.subject is None
 
         adapter = component.queryMultiAdapter((c2, gen), IConnect)
 
-        adapter.connect(gen.head, gen.head.x, gen.head.y)
+        adapter.connect(gen.head)
 
         assert gen.head.connected_to is c2
         assert gen.subject is not None
         
         # Diagram, Class x2, Property *2, Association, LiteralSpec *2
-        assert len(list(UML.select())) == 8, list(UML.select())
+        assert len(list(self.element_factory.select())) == 8, list(self.element_factory.select())
         assert gen.head_end.subject is not None
         assert gen.tail_end.subject is not None
         assert gen.head_end._name_bounds.width == 10
@@ -447,27 +428,24 @@ class ConnectorTestCase(unittest.TestCase):
 
         adapter.disconnect(gen.head)
 
-        assert len(list(UML.select())) == 3, list(UML.select())
+        assert len(list(self.element_factory.select())) == 3, list(self.element_factory.select())
 
     def test_flow_activitynodes(self):
-        assert len(list(UML.select())) == 0
+        flow = self.create(items.FlowItem)
+        i1 = self.create(items.InitialNodeItem, UML.InitialNode)
+        f1 = self.create(items.ActivityFinalNodeItem, UML.ActivityFinalNode)
+        f2 = self.create(items.FlowFinalNodeItem, UML.FlowFinalNode)
 
-        diagram = UML.create(UML.Diagram)
-        flow = diagram.create(items.FlowItem)
-        i1 = diagram.create(items.InitialNodeItem, subject=UML.create(UML.InitialNode))
-        f1 = diagram.create(items.ActivityFinalNodeItem, subject=UML.create(UML.ActivityFinalNode))
-        f2 = diagram.create(items.FlowFinalNodeItem, subject=UML.create(UML.FlowFinalNode))
-
-        assert len(UML.lselect()) == 4
+        assert len(self.element_factory.lselect()) == 4
 
         # head may not connect to FinalNode
 
         adapter = component.queryMultiAdapter((f1, flow), IConnect)
         assert adapter
-        adapter.connect(flow.head, flow.head.x, flow.head.y)
+        adapter.connect(flow.head)
         assert flow.head.connected_to is None
 
-        adapter.connect(flow.tail, flow.tail.x, flow.tail.y)
+        adapter.connect(flow.tail)
         assert flow.head.connected_to is None
         assert flow.tail.connected_to is f1
 
@@ -477,10 +455,10 @@ class ConnectorTestCase(unittest.TestCase):
 
         adapter = component.queryMultiAdapter((f2, flow), IConnect)
         assert adapter
-        adapter.connect(flow.head, flow.head.x, flow.head.y)
+        adapter.connect(flow.head)
         assert flow.head.connected_to is None
 
-        adapter.connect(flow.tail, flow.tail.x, flow.tail.y)
+        adapter.connect(flow.tail)
         assert flow.head.connected_to is None
         assert flow.tail.connected_to is f2
 
@@ -492,43 +470,41 @@ class ConnectorTestCase(unittest.TestCase):
 
         adapter = component.queryMultiAdapter((i1, flow), IConnect)
         assert adapter
-        adapter.connect(flow.tail, flow.tail.x, flow.tail.y)
+        adapter.connect(flow.tail)
         assert flow.head.connected_to is None
         assert flow.tail.connected_to is None
 
-        adapter.connect(flow.head, flow.head.x, flow.head.y)
+        adapter.connect(flow.head)
         assert flow.tail.connected_to is None
         assert flow.head.connected_to is i1
 
     
     def test_flow_action(self):
-        assert len(list(UML.select())) == 0
+        flow = self.create(items.FlowItem)
+        a1 = self.create(items.ActionItem, UML.Action)
+        a2 = self.create(items.ActionItem, UML.Action)
+        o1 = self.create(items.ObjectNodeItem, UML.ObjectNode)
 
-        diagram = UML.create(UML.Diagram)
-        flow = diagram.create(items.FlowItem)
-        a1 = diagram.create(items.ActionItem, subject=UML.create(UML.Action))
-        a2 = diagram.create(items.ActionItem, subject=UML.create(UML.Action))
-        o1 = diagram.create(items.ObjectNodeItem, subject=UML.create(UML.ObjectNode))
-
-        assert len(UML.lselect()) == 5, UML.lselect()
+        # diagram, two actions and object node with literal specification, flow has no subject
+        assert len(self.element_factory.lselect()) == 5, self.element_factory.lselect()
 
         # Connect between two actions (ControlFlow)
         adapter = component.queryMultiAdapter((a1, flow), IConnect)
         assert adapter
-        adapter.connect(flow.tail, flow.tail.x, flow.tail.y)
+        adapter.connect(flow.tail)
 
         assert flow.tail.connected_to is a1
         assert flow.subject is None
 
         adapter = component.queryMultiAdapter((a2, flow), IConnect)
-        adapter.connect(flow.head, flow.head.x, flow.head.y)
+        adapter.connect(flow.head)
 
         assert flow.head.connected_to is a2
         assert flow.tail.connected_to is a1
         assert not flow.subject is None
         assert isinstance(flow.subject, UML.ControlFlow)
 
-        adapter.connect(flow.tail, flow.tail.x, flow.tail.y)
+        adapter.connect(flow.tail)
 
         assert flow.head.connected_to is a2
         assert flow.tail.connected_to is a2
@@ -538,7 +514,7 @@ class ConnectorTestCase(unittest.TestCase):
         # Connection between action and objectNode (ObjectFlow)
 
         adapter = component.queryMultiAdapter((o1, flow), IConnect)
-        adapter.connect(flow.head, flow.head.x, flow.head.y)
+        adapter.connect(flow.head)
 
         assert flow.head.connected_to is o1
         assert flow.tail.connected_to is a2
@@ -552,20 +528,18 @@ class ConnectorTestCase(unittest.TestCase):
         assert flow.subject is None
 
     def test_flow_connect(self):
-        assert len(list(UML.select())) == 0
+        flow1 = self.create(items.FlowItem)
+        flow2 = self.create(items.FlowItem)
+        a1 = self.create(items.ActionItem, UML.Action)
+        a2 = self.create(items.ActionItem, UML.Action)
+        o1 = self.create(items.ObjectNodeItem, UML.ObjectNode)
 
-        diagram = UML.create(UML.Diagram)
-        flow1 = diagram.create(items.FlowItem)
-        flow2 = diagram.create(items.FlowItem)
-        a1 = diagram.create(items.ActionItem, subject=UML.create(UML.Action))
-        a2 = diagram.create(items.ActionItem, subject=UML.create(UML.Action))
-        o1 = diagram.create(items.ObjectNodeItem, subject=UML.create(UML.ObjectNode))
-
-        assert len(UML.lselect()) == 5, UML.lselect()
+        # diagram, two actions and object node, flows has no subjects
+        assert len(self.element_factory.lselect()) == 4, self.element_factory.lselect()
 
         adapter = component.queryMultiAdapter((a1, flow1), IConnect)
         assert adapter
-        adapter.connect(flow1.tail, flow1.tail.x, flow1.tail.y)
+        adapter.connect(flow1.tail)
         assert flow1.tail.connected_to is a1
         assert not a1.subject.incoming, a1.subject.incoming
 
@@ -573,13 +547,13 @@ class ConnectorTestCase(unittest.TestCase):
 
         adapter = component.queryMultiAdapter((a1, flow2), IConnect)
         assert adapter
-        adapter.connect(flow2.tail, flow2.tail.x, flow2.tail.y)
+        adapter.connect(flow2.tail)
         assert flow1.tail.connected_to is a1
         assert flow2.tail.connected_to is a1
 
         adapter = component.queryMultiAdapter((a2, flow1), IConnect)
         assert adapter
-        adapter.connect(flow1.head, flow1.head.x, flow1.head.y)
+        adapter.connect(flow1.head)
         assert flow1.head.connected_to is a2
         assert flow1.tail.connected_to is a1
         assert flow1.subject in a1.subject.incoming
@@ -600,20 +574,17 @@ class ConnectorTestCase(unittest.TestCase):
          [2] The edges coming into and out of a fork node must be either all
              object flows or all control flows.
         """
-        assert len(list(UML.select())) == 0
+        flow1 = self.create(items.FlowItem)
+        flow2 = self.create(items.FlowItem)
+        flow3 = self.create(items.FlowItem)
+        flow4 = self.create(items.FlowItem)
+        a1 = self.create(items.ActionItem, UML.Action)
+        a2 = self.create(items.ActionItem, UML.Action)
+        a3 = self.create(items.ActionItem, UML.Action)
+        a4 = self.create(items.ActionItem, UML.Action)
+        f1 = self.create(itemClass, joinNodeClass)
 
-        diagram = UML.create(UML.Diagram)
-        flow1 = diagram.create(items.FlowItem)
-        flow2 = diagram.create(items.FlowItem)
-        flow3 = diagram.create(items.FlowItem)
-        flow4 = diagram.create(items.FlowItem)
-        a1 = diagram.create(items.ActionItem, subject=UML.create(UML.Action))
-        a2 = diagram.create(items.ActionItem, subject=UML.create(UML.Action))
-        a3 = diagram.create(items.ActionItem, subject=UML.create(UML.Action))
-        a4 = diagram.create(items.ActionItem, subject=UML.create(UML.Action))
-        f1 = diagram.create(itemClass, subject=UML.create(joinNodeClass))
-
-        #assert len(UML.lselect()) == 6, UML.lselect()
+        #assert len(self.element_factory.lselect()) == 6, self.element_factory.lselect()
 
         # Connect between two actions (ControlFlow)
         # Connecting line this:
@@ -625,21 +596,21 @@ class ConnectorTestCase(unittest.TestCase):
 
         adapter = component.queryMultiAdapter((a1, flow1), IConnect)
         assert adapter
-        adapter.connect(flow1.head, flow1.head.x, flow1.head.y)
+        adapter.connect(flow1.head)
 
         adapter = component.queryMultiAdapter((a2, flow2), IConnect)
         assert adapter
-        adapter.connect(flow2.tail, flow2.tail.x, flow2.tail.y)
+        adapter.connect(flow2.tail)
 
         adapter = component.queryMultiAdapter((a3, flow3), IConnect)
         assert adapter
-        adapter.connect(flow3.tail, flow3.tail.x, flow3.tail.y)
+        adapter.connect(flow3.tail)
 
         # Now connect to ForkNode:
 
         adapter = component.queryMultiAdapter((f1, flow1), IConnect)
         assert adapter
-        adapter.connect(flow1.tail, flow1.tail.x, flow1.tail.y)
+        adapter.connect(flow1.tail)
         assert flow1.tail.connected_to is f1
         assert flow1.subject
         assert flow1.subject.target is f1.subject
@@ -648,7 +619,7 @@ class ConnectorTestCase(unittest.TestCase):
 
         adapter = component.queryMultiAdapter((f1, flow2), IConnect)
         assert adapter
-        adapter.connect(flow2.head, flow2.head.x, flow2.head.y)
+        adapter.connect(flow2.head)
         assert flow2.head.connected_to is f1
         assert flow2.subject.source is f1.subject
         assert flow2.subject in f1.subject.outgoing
@@ -657,7 +628,7 @@ class ConnectorTestCase(unittest.TestCase):
 
         adapter = component.queryMultiAdapter((f1, flow3), IConnect)
         assert adapter
-        adapter.connect(flow3.head, flow3.head.x, flow3.head.y)
+        adapter.connect(flow3.head)
         assert flow3.head.connected_to is f1
         assert flow3.subject.source is f1.subject
         assert flow3.subject in f1.subject.outgoing
@@ -669,11 +640,11 @@ class ConnectorTestCase(unittest.TestCase):
 
         adapter = component.queryMultiAdapter((f1, flow4), IConnect)
         assert adapter
-        adapter.connect(flow4.tail, flow4.tail.x, flow4.tail.y)
+        adapter.connect(flow4.tail)
         assert type(f1.subject) is forkNodeClass
 
         adapter = component.queryMultiAdapter((a4, flow4), IConnect)
-        adapter.connect(flow4.head, flow4.head.x, flow4.head.y)
+        adapter.connect(flow4.head)
         assert type(f1.subject) is joinNodeClass
         assert f1.combined
         assert flow4.tail.connected_to is f1
@@ -690,7 +661,7 @@ class ConnectorTestCase(unittest.TestCase):
         # flow4 can be connected as outgoing flow though:
         #adapter = component.queryMultiAdapter((f1, flow4), IConnect)
         #assert adapter
-        #adapter.connect(flow4.head, flow4.head.x, flow4.head.y)
+        #adapter.connect(flow4.head)
         #assert flow4.head.connected_to is f1
 
         adapter.disconnect(flow4.head)
@@ -714,28 +685,28 @@ class ConnectorTestCase(unittest.TestCase):
 
         # Let's try if we can connect both ends of flow2 to the ForkNode:
         adapter = component.queryMultiAdapter((f1, flow2), IConnect)
-        adapter.connect(flow2.tail, flow2.tail.x, flow2.tail.y)
+        adapter.connect(flow2.tail)
         assert flow2.tail.connected_to is None, flow2.tail.connected_to
 
         adapter.disconnect(flow2.head)
-        assert len(f1.subject.incoming) == 1
-        assert len(f1.subject.outgoing) == 1
+        assert len(f1.subject.incoming) == 1, len(f1.subject.incoming)
+        assert len(f1.subject.outgoing) == 0, len(f1.subject.outgoing)
 
         adapter = component.queryMultiAdapter((a2, flow2), IConnect)
-        adapter.connect(flow2.head, flow2.head.x, flow2.head.y)
+        adapter.connect(flow2.head)
         assert len(a2.subject.outgoing) == 0
 
         adapter = component.queryMultiAdapter((f1, flow2), IConnect)
-        adapter.connect(flow2.tail, flow2.tail.x, flow2.tail.y)
+        adapter.connect(flow2.tail)
         assert len(a2.subject.outgoing) == 1
         assert len(f1.subject.incoming) == 2
-        assert len(f1.subject.outgoing) == 1
+        assert len(f1.subject.outgoing) == 0, len(f1.subject.outgoing)
         assert type(f1.subject) is joinNodeClass, f1.subject
 
         # And of course I can't add another outgoing edge:
         #adapter = component.queryMultiAdapter((f1, flow4), IConnect)
         #assert adapter
-        #adapter.connect(flow4.head, flow4.head.x, flow4.head.y)
+        #adapter.connect(flow4.head)
         #assert flow4.head.connected_to is None
 
 
@@ -747,4 +718,222 @@ class ConnectorTestCase(unittest.TestCase):
                                      forkNodeClass=UML.DecisionNode,
                                      joinNodeClass=UML.MergeNode)
 
+
+    def test_message_connect_lost(self):
+        """Test lost message creating
+        """
+        factory = self.element_factory
+        lifeline = self.create(items.LifelineItem)
+        message = self.create(items.MessageItem)
+
+        assert message.subject is None
+
+        adapter = component.queryMultiAdapter((lifeline, message), IConnect)
+
+        assert adapter is not None
+        
+        # connect tail of message to lifeline.
+        adapter.connect(message.head)
+
+        # If one side is connected a "lost" message is created
+        assert message.subject is not None
+        assert len(factory.lselect(lambda e: e.isKindOf(UML.Message))) == 1
+        assert len(factory.lselect(lambda e: e.isKindOf(UML.EventOccurrence))) == 1
+        assert factory.lselect(lambda e: e.isKindOf(UML.Message))[0] is message.subject
+        assert factory.lselect(lambda e: e.isKindOf(UML.EventOccurrence))[0] is message.subject.sendEvent
+        
+        adapter.disconnect(message.head)
+        assert message.subject is None
+        assert len(factory.lselect(lambda e: e.isKindOf(UML.Message))) == 0, \
+                factory.lselect(lambda e: e.isKindOf(UML.Message))
+
+
+    def test_message_connect(self):
+        """Test message connection (not lost, not found)
+        """
+        factory = self.element_factory
+
+        lifeline1 = self.create(items.LifelineItem)
+        lifeline2 = self.create(items.LifelineItem)
+
+        message = self.create(items.MessageItem)
+        assert message.subject is None
+
+        adapter = component.queryMultiAdapter((lifeline1, message), IConnect)
+        assert adapter is not None
+        
+        # connect tail of message to lifeline.
+        adapter.connect(message.head)
+
+        # If one side is connected a "lost" message is created
+        assert message.subject is not None
+        assert len(factory.lselect(lambda e: e.isKindOf(UML.Message))) == 1
+        assert len(factory.lselect(lambda e: e.isKindOf(UML.EventOccurrence))) == 1
+        assert factory.lselect(lambda e: e.isKindOf(UML.Message))[0] is message.subject
+        assert factory.lselect(lambda e: e.isKindOf(UML.EventOccurrence))[0] is message.subject.sendEvent
+        
+        adapter = component.queryMultiAdapter((lifeline2, message), IConnect)
+        assert adapter is not None
+
+        adapter.connect(message.tail)
+        assert len(factory.lselect(lambda e: e.isKindOf(UML.Message))) == 1
+        assert len(factory.lselect(lambda e: e.isKindOf(UML.EventOccurrence))) == 2
+        assert factory.lselect(lambda e: e.isKindOf(UML.Message))[0] is message.subject
+        assert message.subject.sendEvent in factory.lselect(lambda e: e.isKindOf(UML.EventOccurrence))
+        assert message.subject.receiveEvent in factory.lselect(lambda e: e.isKindOf(UML.EventOccurrence))
+        
+        adapter = component.queryMultiAdapter((lifeline1, message), IConnect)
+        adapter.disconnect(message.head)
+        assert len(factory.lselect(lambda e: e.isKindOf(UML.Message))) == 1
+        assert len(factory.lselect(lambda e: e.isKindOf(UML.EventOccurrence))) == 1
+        
+        adapter = component.queryMultiAdapter((lifeline2, message), IConnect)
+        adapter.disconnect(message.tail)
+
+        assert message.subject is None
+        assert len(factory.lselect(lambda e: e.isKindOf(UML.Message))) == 0, \
+                factory.lselect(lambda e: e.isKindOf(UML.Message))
+        assert len(factory.lselect(lambda e: e.isKindOf(UML.EventOccurrence))) == 0, \
+                factory.lselect(lambda e: e.isKindOf(UML.EventOccurrence))
+
+
+    def test_message_connect_cd(self):
+        """Test connecting message on communication diagram
+        """
+        lifeline1 = self.create(items.LifelineItem)
+        lifeline2 = self.create(items.LifelineItem)
+
+        # make second lifeline to be on sequence diagram
+        lifetime = lifeline2.lifetime
+        lifetime.bottom.y += 10
+        assert lifetime.is_visible
+
+        message = self.create(items.MessageItem)
+        assert message.subject is None
+
+        adapter = component.queryMultiAdapter((lifeline1, message), IConnect)
+        assert adapter is not None
+        
+        # connect head of message to lifeline
+        adapter.connect(message.head)
+
+        adapter = component.queryMultiAdapter((lifeline2, message), IConnect)
+        assert adapter is not None
+
+        adapter.connect(message.tail)
+        # we should not be connected to second lifeline as it is on
+        # sequence diagram
+        assert message.tail.connected_to is None
+
+        # make lifetime invisible and connect again
+        lifetime.bottom.y -= 10
+        assert not lifetime.is_visible
+
+        adapter.connect(message.tail)
+        assert message.tail.connected_to is lifeline2
+
+
+    def test_message_connect_sd(self):
+        """Test connecting message on sequence diagram
+        """
+        lifeline1 = self.create(items.LifelineItem)
+        lifeline2 = self.create(items.LifelineItem)
+
+        # make first lifeline to be on sequence diagram
+        lifetime = lifeline1.lifetime
+        lifetime.bottom.y += 10
+        assert lifetime.is_visible
+
+        message = self.create(items.MessageItem)
+        assert message.subject is None
+
+        adapter = component.queryMultiAdapter((lifeline1, message), IConnect)
+        assert adapter is not None
+        
+        # connect head of message to lifeline
+        adapter.connect(message.head)
+
+        adapter = component.queryMultiAdapter((lifeline2, message), IConnect)
+        assert adapter is not None
+
+        adapter.connect(message.tail)
+        # we should not be connected to second lifeline as it is on
+        # communication diagram
+        assert message.tail.connected_to is None
+
+        # make second lifeline to be on sequence diagram
+        lifetime = lifeline2.lifetime
+        lifetime.bottom.y += 10
+        assert lifetime.is_visible
+        
+        # connect again
+        adapter.connect(message.tail)
+        assert message.tail.connected_to is lifeline2
+
+
+    def test_messages_disconnect_cd(self):
+        """Test disconnecting messages on communication diagram
+        """
+        factory = self.element_factory
+
+        lifeline1 = self.create(items.LifelineItem)
+        lifeline2 = self.create(items.LifelineItem)
+
+        message = self.create(items.MessageItem)
+        assert message.subject is None
+
+        adapter = component.queryMultiAdapter((lifeline1, message), IConnect)
+        assert adapter is not None
+        
+        # connect head of message to lifeline
+        adapter.connect(message.head)
+
+        adapter = component.queryMultiAdapter((lifeline2, message), IConnect)
+        assert adapter is not None
+
+        adapter.connect(message.tail)
+        assert message.tail.connected_to is lifeline2
+        
+        assert len(factory.lselect(lambda e: e.isKindOf(UML.Message))) == 1
+        assert len(factory.lselect(lambda e: e.isKindOf(UML.EventOccurrence))) == 2
+        assert factory.lselect(lambda e: e.isKindOf(UML.Message))[0] is message.subject
+        assert message.subject.sendEvent in factory.lselect(lambda e: e.isKindOf(UML.EventOccurrence))
+        assert message.subject.receiveEvent in factory.lselect(lambda e: e.isKindOf(UML.EventOccurrence))
+
+        subject = message.subject
+
+        # add some more messages
+        m1 = factory.create(UML.Message)
+        m1.sendEvent = subject.sendEvent
+        m1.receiveEvent = subject.receiveEvent
+
+        m2 = factory.create(UML.Message)
+        m2.sendEvent = subject.sendEvent
+        m2.receiveEvent = subject.receiveEvent
+
+        message.add_message(m1, False)
+        message.add_message(m2, False)
+
+        # add some inverted messages
+        m3 = factory.create(UML.Message)
+        m3.sendEvent = subject.receiveEvent
+        m3.receiveEvent = subject.sendEvent
+
+        m4 = factory.create(UML.Message)
+        m4.sendEvent = subject.receiveEvent
+        m4.receiveEvent = subject.sendEvent
+
+        message.add_message(m3, True)
+        message.add_message(m4, True)
+
+        assert len(factory.lselect(lambda e: e.isKindOf(UML.Message))) == 5
+
+        # disconnect
+        adapter.disconnect(message.head)
+        adapter.disconnect(message.tail)
+
+        # we expect no messages
+        assert len(factory.lselect(lambda e: e.isKindOf(UML.Message))) == 0
+
+
 # vim:sw=4:et:ai
