@@ -7,7 +7,6 @@ from math import pi
 from gaphor import UML
 from gaphor.diagram.nameditem import NamedItem
 from gaphor.diagram.style import ALIGN_CENTER, ALIGN_MIDDLE
-from gaphas.util import text_align, text_extents
 
 class ActionItem(NamedItem):
     __uml__   = UML.Action
@@ -16,15 +15,13 @@ class ActionItem(NamedItem):
         'name-align': (ALIGN_CENTER, ALIGN_MIDDLE),
     }
 
-    def pre_update(self, context):
-        self.update_name_size(context)
-        self.min_width, self.min_height = self.get_name_size()
-        super(ActionItem, self).pre_update(context)
 
     def draw(self, context):
         """
         Draw action symbol.
         """
+        super(ActionItem, self).draw(context)
+
         c = context.cairo
 
         d = 15
@@ -41,7 +38,5 @@ class ActionItem(NamedItem):
 
         c.stroke()
 
-        super(ActionItem, self).draw(context)
-
 
 # vim:sw=4:et
