@@ -4,16 +4,22 @@ Activity control nodes.
 
 import math
 
-from gaphas.util import path_ellipse, text_align
+from gaphas.util import path_ellipse
 from gaphas.state import observed, reversible_property
+from gaphas.item import Handle, Item
+from gaphas.constraint import EqualsConstraint, LessThanConstraint
+from gaphas.geometry import distance_line_point
 
 from gaphor import UML
-from gaphor import resource
+from gaphor.core import inject
+from gaphor.diagram.diagramitem import DiagramItem
 from gaphor.diagram.nameditem import NamedItem
 from gaphor.diagram.style import ALIGN_LEFT, ALIGN_CENTER, ALIGN_TOP, \
         ALIGN_RIGHT, ALIGN_BOTTOM
 from gaphor.diagram.style import get_text_point
-from gaphas.util import text_extents
+
+
+DEFAULT_JOIN_SPEC = 'and'
 
 
 class ActivityNodeItem(NamedItem):
@@ -119,13 +125,17 @@ class FlowFinalNodeItem(ActivityNodeItem):
         
 
 
-class ForkDecisionNodeItem(ActivityNodeItem):
+class DecisionNodeItem(ActivityNodeItem):
     """
-    Abstract class for fork and decision UI nodes. These nodes contain
-    combined property, which determines if the they represent combination
-    of fork/join or decision/merge nodes as described in UML
-    specification.
+    Representation of decision or merge node.
     """
+    __uml__   = UML.DecisionNode
+    __style__   = {
+        'min-size':   (20, 30),
+        'name-align': (ALIGN_LEFT, ALIGN_TOP),
+    }
+
+    RADIUS = 15
 
     def __init__(self, id=None):
         ActivityNodeItem.__init__(self, id)
@@ -135,13 +145,13 @@ class ForkDecisionNodeItem(ActivityNodeItem):
     def save(self, save_func):
         if self._combined:
             save_func('combined', self._combined, reference=True)
-        super(ForkDecisionNodeItem, self).save(save_func)
+        super(DecisionNodeItem, self).save(save_func)
 
     def load(self, name, value):
         if name == 'combined':
             self._combined = value
         else:
-            super(ForkDecisionNodeItem, self).load(name, value)
+            super(DecisionNodeItem, self).load(name, value)
 
     @observed
     def _set_combined(self, value):
@@ -150,20 +160,6 @@ class ForkDecisionNodeItem(ActivityNodeItem):
 
     combined = reversible_property(lambda s: s._combined, _set_combined)
         
-
-class DecisionNodeItem(ForkDecisionNodeItem):
-    """
-    Representation of decision or merge node.
-    """
-
-    __uml__   = UML.DecisionNode
-    __style__   = {
-        'min-size':   (20, 30),
-        'name-align': (ALIGN_LEFT, ALIGN_TOP),
-    }
-
-    RADIUS = 15
-
     def draw(self, context):
         """
         Draw diamond shape, which represents decision and merge nodes.
@@ -183,55 +179,144 @@ class DecisionNodeItem(ForkDecisionNodeItem):
 
 
 
-class ForkNodeItem(ForkDecisionNodeItem):
+class ForkNodeItem(Item, DiagramItem):
     """
     Representation of fork and join node.
     """
-    __uml__   = UML.JoinNode
-    #__uml__   = UML.ForkNode
+    __namedelement__ = True
+
+    element_factory = inject('element_factory')
+
+    __uml__   = UML.ForkNode
+
     __style__ = {
         'min-size':   (6, 45),
         'name-align': (ALIGN_CENTER, ALIGN_BOTTOM),
+        'name-padding': (2, 2, 2, 2),
+        'name-outside': True,
+        'name-align-str': None,
+    }
+
+    STYLE_TOP = {
+        'text-align': (ALIGN_CENTER, ALIGN_TOP),
+        'text-outside': True,
     }
 
     def __init__(self, id=None):
-        ForkDecisionNodeItem.__init__(self, id)
+        Item.__init__(self)
+        DiagramItem.__init__(self, id)
+        
+        self._handles.extend((Handle(), Handle()))
 
-        self._join_spec = 'join spec test'
-        self._join_spec_x = 0
-        self._join_spec_y = 0
+        self._combined = None
+        self._constraints = []
 
-    def update(self, context):
+        self._join_spec = self.add_text('joinSpec.value',
+            pattern='{ joinSpec = %s }',
+            style=self.STYLE_TOP,
+            visible=self.is_join_spec_visible)
+
+
+    def save(self, save_func):
+        save_func('matrix', tuple(self.matrix))
+        save_func('height', float(self._handles[1].y))
+        if self._combined:
+            save_func('combined', self._combined, reference=True)
+        DiagramItem.save(self, save_func)
+
+    def load(self, name, value):
+        if name == 'matrix':
+            self.matrix = eval(value)
+        elif name == 'height':
+            self._handles[1].y = eval(value)
+        elif name == 'combined':
+            self._combined = value
+        else:
+            #DiagramItem.load(self, name, value)
+            super(ForkNodeItem, self).load(name, value)
+
+    def postload(self):
+        subject = self.subject
+        if subject and isinstance(subject, UML.JoinNode) and subject.joinSpec:
+            self._join_spec.text = self.subject.joinSpec.value
+        super(ForkNodeItem, self).postload()
+
+    @observed
+    def _set_combined(self, value):
+        #self.preserve_property('combined')
+        self._combined = value
+
+    combined = reversible_property(lambda s: s._combined, _set_combined)
+
+    def setup_canvas(self):
+        Item.setup_canvas(self)
+
+        h1, h2 = self._handles
+        cadd = self.canvas.solver.add_constraint
+        c1 = EqualsConstraint(a=h1.x, b=h2.x)
+        c2 = LessThanConstraint(smaller=h1.y, bigger=h2.y, delta=30)
+        self._constraints.extend((cadd(c1), cadd(c2)))
+
+
+    def teardown_canvas(self):
+        super(ForkNodeItem, self).teardown_canvas()
+        for c in self._constraints:
+            self.canvas.solver.remove_constraint(c)
+
+
+    def is_join_spec_visible(self):
         """
-        Update join specification position.
+        Check if join specification should be displayed.
         """
-        if isinstance(self.subject, UML.JoinNode):
-            self._join_spec_x, self._join_spec_y = get_text_point(
-                    text_extents(context.cairo, self.subject.joinSpec.value),
-                    self.width, self.height,
-                    (ALIGN_CENTER, ALIGN_TOP),
-                    (10, 0, 0, 0),
-                    True)
-        super(ForkNodeItem, self).update(context)
+        return isinstance(self.subject, UML.JoinNode) \
+            and self.subject.joinSpec is not None \
+            and self.subject.joinSpec.value != DEFAULT_JOIN_SPEC
+
+
+    def text_align(self, extents, align, padding, outside):
+        h1, h2 = self._handles
+        w, _ = self.style.min_size
+        h = h2.y - h1.y
+        x, y = get_text_point(extents, w, h, align, padding, outside)
+
+        return x, y
+
+
+    def pre_update(self, context):
+        self.update_stereotype()
+        Item.pre_update(self, context)
+        DiagramItem.pre_update(self, context)
+
+
+    def post_update(self, context):
+        Item.post_update(self, context)
+        DiagramItem.post_update(self, context)
+
 
     def draw(self, context):
         """
         Draw vertical line - symbol of fork and join nodes. Join
         specification is also drawn above the item.
         """
-        cr = context.cairo
-        cr.set_line_width(self.width)
-        x = self.width / 2.
-        cr.move_to(x, 0)
-        cr.line_to(x, self.height)
-        cr.move_to(self.name_x, self.name_y)
+        Item.draw(self, context)
+        DiagramItem.draw(self, context)
 
-        if isinstance(self.subject, UML.JoinNode):
-            text_align(cr, self._join_spec_x, self._join_spec_y,
-                       self.subject.joinSpec.value, align_x=1, align_y=1)
+        cr = context.cairo
+
+        cr.set_line_width(6)
+        h1, h2 = self._handles
+        cr.move_to(h1.x, h1.y)
+        cr.line_to(h2.x, h2.y)
 
         cr.stroke()
-        super(ForkNodeItem, self).draw(context)
+
+
+    def point(self, x, y):
+        h1, h2 = self._handles
+        d, p = distance_line_point(h1.pos, h2.pos, (x, y))
+        # Substract line_width / 2
+        return d - 3
+
 
     def on_subject_notify(self, pspec, notifiers = ()):
         """
@@ -240,14 +325,49 @@ class ForkNodeItem(ForkDecisionNodeItem):
         If subject is join node, then set subject of join specification
         text element.
         """
-        ForkDecisionNodeItem.on_subject_notify(self, pspec, notifiers)
-        if self.subject and isinstance(self.subject, UML.JoinNode):
-            if not self.subject.joinSpec:
-                self.subject.joinSpec = UML.create(UML.LiteralSpecification)
-                self.subject.joinSpec.value = 'and'
-            #self._join_spec.subject = self.subject.joinSpec
-        #else:
-        #    self._join_spec.subject = None
+        subject = self.subject
+        if is_join_node(subject):
+            join_spec_notifiers = ('joinSpec', 'joinSpec.value')
+        else:
+            join_spec_notifiers = ()
+
+        DiagramItem.on_subject_notify(self, pspec, join_spec_notifiers + notifiers)
+
+        if is_join_node(subject) and not (subject.joinSpec and subject.joinSpec.value):
+            self.set_join_spec(DEFAULT_JOIN_SPEC)
         self.request_update()
 
+
+    def set_join_spec(self, value):
+        """
+        Set join specification.
+        """
+        subject = self.subject
+        if not is_join_node(subject):
+            return
+
+        if not subject.joinSpec:
+            subject.joinSpec = self.element_factory.create(UML.LiteralSpecification)
+
+        if not value:
+            value = DEFAULT_JOIN_SPEC
+
+        subject.joinSpec.value = value
+        self._join_spec.text = value
+
+
+    def on_subject_notify__joinSpec(self, subject, pspec=None):
+        self.request_update()
+
+
+    def on_subject_notify__joinSpec_value(self, subject, pspec=None):
+        self.request_update()
+
+
+def is_join_node(subject):
+    """
+    Check if ``subject`` is join node. 
+    """
+    return subject and isinstance(subject, UML.JoinNode)
+
 # vim:sw=4:et
