@@ -13,86 +13,28 @@ Plan:
 # are connected to the same Class, the head_end property is connected to the
 # tail end and visa versa.
 
-from math import atan2, pi
-
 from gaphas.util import text_extents, text_align, text_multiline
 from gaphas.state import reversible_property
 from gaphas import Item
-from gaphas.geometry import Rectangle
+from gaphas.geometry import Rectangle, distance_point_point_fast
 from gaphas.geometry import distance_rectangle_point, distance_line_point
 
-from gaphor import resource, UML
-#from gaphor.diagram import Relationship
+from gaphor import UML
 from gaphor.diagram.diagramitem import SubjectSupport
-from gaphor.diagram.diagramline import DiagramLine
-
-#class AssociationRelationship(Relationship):
-#    """Relationship for associations.
-#    """
-def xxx():
-    def relationship(self, line, head_subject = None, tail_subject = None):
-        # First check if we do not already contain the right subject:
-        if line.subject:
-            end1 = line.subject.memberEnd[0]
-            end2 = line.subject.memberEnd[1]
-            if (end1.type is head_type and end2.type is tail_type) \
-               or (end2.type is head_type and end1.type is tail_type):
-                return
-                
-        # Find all associations and determine if the properties on the
-        # association ends have a type that points to the class.
-        Association = UML.Association
-        for assoc in resource(UML.ElementFactory).itervalues():
-            if isinstance(assoc, Association):
-                #print 'assoc.memberEnd', assoc.memberEnd
-                end1 = assoc.memberEnd[0]
-                end2 = assoc.memberEnd[1]
-                if (end1.type is head_type and end2.type is tail_type) \
-                   or (end2.type is head_type and end1.type is tail_type):
-                    # check if this entry is not yet in the diagram
-                    # Return if the association is not (yet) on the canvas
-                    for item in assoc.presentation:
-                        if item.canvas is line.canvas:
-                            break
-                    else:
-                        return assoc
-        return None
+from gaphor.diagram.diagramline import NamedLine
 
 
-
-class AssociationItem(DiagramLine):
-    """AssociationItem represents associations. 
+class AssociationItem(NamedLine):
+    """
+    AssociationItem represents associations. 
     An AssociationItem has two AssociationEnd items. Each AssociationEnd item
     represents a Property (with Property.association == my association).
     """
 
     __uml__ = UML.Association
 
-    association_popup_menu = (
-        'separator',
-        'AssociationShowDirection',
-        'AssociationInvertDirection',
-        'separator',
-        'Head', (
-            'Head_unknownNavigation',
-            'Head_isNotNavigable',
-            'Head_isNavigable',
-            'separator',
-            'Head_AggregationNone',
-            'Head_AggregationShared',
-            'Head_AggregationComposite'),
-        'Tail', (
-            'Tail_unknownNavigation',
-            'Tail_isNotNavigable',
-            'Tail_isNavigable',
-            'separator',
-            'Tail_AggregationNone',
-            'Tail_AggregationShared',
-            'Tail_AggregationComposite'),
-    )
-
     def __init__(self, id=None):
-        DiagramLine.__init__(self, id)
+        NamedLine.__init__(self, id)
 
         # AssociationEnds are really inseperable from the AssociationItem.
         # We give them the same id as the association item.
@@ -121,7 +63,7 @@ class AssociationItem(DiagramLine):
         del self._tail_end._canvas
 
     def save(self, save_func):
-        DiagramLine.save(self, save_func)
+        NamedLine.save(self, save_func)
         save_func('show-direction', self._show_direction)
         if self._head_end.subject:
             save_func('head-subject', self._head_end.subject)
@@ -139,10 +81,10 @@ class AssociationItem(DiagramLine):
             #self._tail_end.load('subject', value)
             self._tail_end.subject = value
         else:
-            DiagramLine.load(self, name, value)
+            NamedLine.load(self, name, value)
 
     def postload(self):
-        DiagramLine.postload(self)
+        NamedLine.postload(self)
         self._head_end.set_text()
         self._tail_end.set_text()
 
@@ -153,13 +95,7 @@ class AssociationItem(DiagramLine):
     def unlink(self):
         self._head_end.unlink()
         self._tail_end.unlink()
-        DiagramLine.unlink(self)
-
-    def get_popup_menu(self):
-        if self.subject:
-            return self.popup_menu + self.association_popup_menu
-        else:
-            return self.popup_menu
+        NamedLine.unlink(self)
 
     def invert_direction(self):
         """Invert the direction of the association, this is done by
@@ -168,17 +104,12 @@ class AssociationItem(DiagramLine):
         if not self.subject:
             return
 
-        self.subject.memberEnd.moveDown(self.subject.memberEnd[0])
+        self.subject.memberEnd.swap(self.subject.memberEnd[0], self.subject.memberEnd[1])
         self.request_update()
 
     def on_subject_notify(self, pspec, notifiers=()):
-        DiagramLine.on_subject_notify(self, pspec,
-                                           notifiers + ('name', 'ownedEnd', 'memberEnd'))
-
-        self.on_subject_notify__name(self.subject, pspec)
-
-    def on_subject_notify__name(self, subject, pspec):
-        self.request_update()
+        NamedLine.on_subject_notify(self, pspec,
+                notifiers + ('ownedEnd', 'memberEnd'))
 
     def on_subject_notify__ownedEnd(self, subject, pspec):
         self.request_update()
@@ -186,7 +117,7 @@ class AssociationItem(DiagramLine):
     def on_subject_notify__memberEnd(self, subject, pspec):
         self.request_update()
 
-    def update(self, context):
+    def post_update(self, context):
         """
         Update the shapes and sub-items of the association.
         """
@@ -222,30 +153,25 @@ class AssociationItem(DiagramLine):
                 self.draw_tail = self.draw_tail_undefined
 
             if self._show_direction:
-                m = len(self._handles) / 2
-                h0, h1 = self._handles[m - 1], self._handles[m]
-                self._dir_angle = atan2(h1.y - h0.y, h1.x - h0.x)
-                self._dir_pos = (h0.x + h1.x) / 2, (h0.y + h1.y) / 2
-                if self.tail_end.subject is self.subject.memberEnd[0]:
-                    self._dir_angle += pi
+                inverted = self.tail_end.subject is self.subject.memberEnd[0]
+                pos, angle = self._get_center_pos(inverted)
+                self._dir_pos = pos
+                self._dir_angle = angle
         else:
             self.draw_head = self.draw_head_undefined
             self.draw_tail = self.draw_tail_undefined
 
         # update relationship after self.set calls to avoid circural updates
-        DiagramLine.update(self, context)
+        NamedLine.post_update(self, context)
 
         # Calculate alignment of the head name and multiplicity
-        self._head_end.update(context, handles[0].pos,
+        self._head_end.post_update(context, handles[0].pos,
                                      handles[1].pos)
 
         # Calculate alignment of the tail name and multiplicity
-        self._tail_end.update(context, handles[-1].pos,
+        self._tail_end.post_update(context, handles[-1].pos,
                                      handles[-2].pos)
         
-        # update name label:
-        self._label_bounds = self.update_label(context, self.subject and self.subject.name)
-
 
     def point(self, x, y):
         """Returns the distance from the Association to the (mouse) cursor.
@@ -366,13 +292,15 @@ class AssociationItem(DiagramLine):
             finally:
                 cr.restore()
 
-        if self.subject and self.subject.name:
-            #cr.move_to(self._label_bounds[0], self._label_bounds[1])
-            #cr.show_text(self.subject.name or '')
-            text_align(cr, self._label_bounds[0], self._label_bounds[1],
-                       self.subject.name or '', align_x=1, align_y=1)
-
 
+    def item_at(self, x, y):
+        if distance_point_point_fast(self._handles[0].pos, (x, y)) < 10:
+            return self._head_end
+        elif distance_point_point_fast(self._handles[-1].pos, (x, y)) < 10:
+            return self._tail_end
+        return self
+        
+        
 class AssociationEnd(SubjectSupport):
     """
     An association end represents one end of an association. An association
@@ -492,8 +420,9 @@ class AssociationEnd(SubjectSupport):
         return navigability
                 
 
-    def set_navigable(self, navigable):
-        """Change the AssociationEnd's navigability.
+    def _set_navigability(self, navigable):
+        """
+        Change the AssociationEnd's navigability.
 
         A warning is issued if the subject or opposite property is missing.
         """
@@ -540,6 +469,7 @@ class AssociationEnd(SubjectSupport):
             log.warning('AssociationEnd.set_navigable: %s missing' % \
                         (subject and 'subject' or 'opposite Property'))
 
+    navigability = property(get_navigability, _set_navigability)
 
     def point_name(self, x, y):
         p = (x, y)
@@ -560,7 +490,7 @@ class AssociationEnd(SubjectSupport):
     def get_mult(self):
         return self._mult
 
-    def update(self, context, p1, p2):
+    def post_update(self, context, p1, p2):
         """
         Update label placement for association's name and
         multiplicity label. p1 is the line end and p2 is the last
@@ -715,10 +645,10 @@ class AssociationEnd(SubjectSupport):
         if context.hovered or context.focused or context.draw_all:
             cr.set_line_width(0.5)
             b = self._name_bounds
-            cr.rectangle(b.x0, b.y0, b.width, b.height)
+            cr.rectangle(b.x, b.y, b.width, b.height)
             cr.stroke()
             b = self._mult_bounds
-            cr.rectangle(b.x0, b.y0, b.width, b.height)
+            cr.rectangle(b.x, b.y, b.width, b.height)
             cr.stroke()
     
 
