@@ -15,12 +15,6 @@ class CommentItem(ElementItem):
     EAR=15
     OFFSET=5
 
-    popup_menu = (
-        'EditItem',
-        'separator',
-        'EditDelete',
-    )
-
     def __init__(self, id=None):
         ElementItem.__init__(self, id)
         self.min_width = CommentItem.EAR + 2 * CommentItem.OFFSET
@@ -32,16 +26,13 @@ class CommentItem(ElementItem):
         pass
 
     def on_subject_notify(self, pspec):
-        """See DiagramItem.on_subject_notify()."""
+        """
+        See DiagramItem.on_subject_notify().
+        """
         ElementItem.on_subject_notify(self, pspec, ('body',))
-
-        ##if self.subject:
-        #    self.on_subject_notify__body(self.subject, None)
         self.request_update()
 
     def on_subject_notify__body(self, subject, pspec):
-        print 'on_subject_notify__body: %s' % self.subject.body
-        #self.sbody.set_text(self.subject.body or '')
         self.request_update()
 
     # DiaCanvasItem callbacks:
@@ -51,11 +42,11 @@ class CommentItem(ElementItem):
         cr = context.cairo
         w, h = text_extents(cr, self.subject.body, multiline=True, padding=2)
         self.min_width = w + 10
-        self.min_height = h + 20
+        self.min_height = h + 10
         ElementItem.pre_update(self, context)
 
-    def update(self, context):
-        ElementItem.update(self, context)
+    def post_update(self, context):
+        ElementItem.post_update(self, context)
 
     def draw(self, context):
         if not self.subject: return
