@@ -1,18 +1,18 @@
-# vim:sw=4:et
-"""DiagramItem provides basic functionality for presentations.
+"""
+DiagramItem provides basic functionality for presentations.
 Such as a modifier 'subject' property and a unique id.
 """
 
-from gaphor import resource
 from gaphor import UML
 from gaphor.misc import uniqueid
-from gaphor.UML import Element, Presentation
 from gaphor.diagram import DiagramItemMeta
+from gaphor.diagram.textelement import EditableTextSupport
+from gaphor.diagram.style import ALIGN_CENTER, ALIGN_TOP
 
 STEREOTYPE_OPEN  = '\xc2\xab' # '<<'
 STEREOTYPE_CLOSE = '\xc2\xbb' # '>>'
 
-class SubjectSupport(Presentation, Element):
+class SubjectSupport(UML.Presentation):
     """
     Support class that adds support methods for Presentation.subject.
 
@@ -21,8 +21,7 @@ class SubjectSupport(Presentation, Element):
     """
 
     def __init__(self):
-        Presentation.__init__(self)
-        Element.__init__(self)
+        UML.Presentation.__init__(self)
         # Add the class' on_subject_notify() as handler:
         self.connect('subject', type(self).on_subject_notify)
 
@@ -31,7 +30,8 @@ class SubjectSupport(Presentation, Element):
         self.__the_subject = None
 
     def _subject_connect_helper(self, element, callback_prefix, prop_list):
-        """Connect a signal notifier. The notifier can be just the name of
+        """
+        Connect a signal notifier. The notifier can be just the name of
         one of the subjects properties.
 
         See: DiagramItem.on_subject_notify()
@@ -136,13 +136,6 @@ class SubjectSupport(Presentation, Element):
                 #self._subject_connect(self.subject, n)
                 subject_connect_helper(subject, callback_prefix, n)
 
-        # Execute some sort of ItemNewSubject action
-        try:
-            main_window = resource('MainWindow')
-        except KeyError:
-            pass
-        else:
-            main_window.execute_action('ItemNewSubject')
         self.request_update()
 
 
@@ -150,9 +143,26 @@ class StereotypeSupport(object):
     """
     Support methods for stereotypes.
     """
+    STEREOTYPE_ALIGN = {
+        'text-align'  : (ALIGN_CENTER, ALIGN_TOP),
+        'text-padding': (5, 10, 2, 10),
+        'text-outside': False,
+        'text-align-group': 'stereotype',
+    }
 
     def __init__(self):
-        self._stereotype = None
+        self._stereotype = self.add_text('stereotype',
+                style=self.STEREOTYPE_ALIGN,
+                pattern='%s%%s%s' % (STEREOTYPE_OPEN, STEREOTYPE_CLOSE),
+                visible=self.is_stereotype_visible)
+
+
+    def is_stereotype_visible(self):
+        """
+        Display stereotype if it is not empty.
+        """
+        return self._stereotype.text
+
 
     def set_stereotype(self, text=None):
         """
@@ -162,10 +172,7 @@ class StereotypeSupport(object):
 
         @arg text: stereotype text
         """
-        if text:
-            self._stereotype = STEREOTYPE_OPEN + text + STEREOTYPE_CLOSE
-        else:
-            self._stereotype = None
+        self._stereotype.text = text
         self.request_update()
 
     stereotype = property(lambda s: s._stereotype, set_stereotype)
@@ -239,7 +246,7 @@ class StereotypeSupport(object):
         return None
 
 
-class DiagramItem(SubjectSupport, StereotypeSupport):
+class DiagramItem(SubjectSupport, StereotypeSupport, EditableTextSupport):
     """
     Basic functionality for all model elements (lines and elements!).
 
@@ -262,11 +269,9 @@ class DiagramItem(SubjectSupport, StereotypeSupport):
 
     __metaclass__ = DiagramItemMeta
 
-    stereotype_list = []
-    popup_menu = ('Stereotype', stereotype_list)
-
     def __init__(self, id=None):
         SubjectSupport.__init__(self)
+        EditableTextSupport.__init__(self)
         StereotypeSupport.__init__(self)
 
         self._id = id
@@ -274,10 +279,13 @@ class DiagramItem(SubjectSupport, StereotypeSupport):
         # properties, which should be saved in file
         self._persistent_props = set()
 
+
     id = property(lambda self: self._id, doc='Id')
 
+
     def set_prop_persistent(self, name):
-        """Specify property of diagram item, which should be saved in file.
+        """
+        Specify property of diagram item, which should be saved in file.
         """
         self._persistent_props.add(name)
 
@@ -308,13 +316,16 @@ class DiagramItem(SubjectSupport, StereotypeSupport):
         if self.subject:
             self.on_subject_notify(type(self).subject)
 
+
     def save_property(self, save_func, name):
-        """Save a property, this is a shorthand method.
+        """
+        Save a property, this is a shorthand method.
         """
         save_func(name, getattr(self, name.replace('-', '_')))
 
     def save_properties(self, save_func, *names):
-        """Save a property, this is a shorthand method.
+        """
+        Save a property, this is a shorthand method.
         """
         for name in names:
             self.save_property(save_func, name)
@@ -324,55 +335,35 @@ class DiagramItem(SubjectSupport, StereotypeSupport):
         Remove the item from the canvas and set subject to None.
         """
         if self.canvas:
-            self.canvas.remove(self)
+            try:
+                self.canvas.remove(self)
+            except KeyError:
+                # Canvas was already removed
+                pass
         self.subject = None
         super(DiagramItem, self).unlink()
 
-    def get_popup_menu(self):
-        """In the popup menu a submenu is created with Stereotypes than can be
-        applied to this classifier (Class, Interface).
-        If the class itself is a metaclass, an option is added to check if the class
-        exists.
-        """
-        subject = self.subject
-        stereotype_list = self.stereotype_list
-        stereotype_list[:] = []
 
-        # UML specs does not allow to extend stereotypes with stereotypes
-        if subject and not isinstance(subject, UML.Stereotype):
-            # look for stereotypes to put them into context menu of an item
-            # this can be only done when subject exists
+    def draw(self, context):
+        EditableTextSupport.draw(self, context)
 
-            from gaphor.actions.itemactions import ApplyStereotypeAction, register_action
-
-            cls = type(subject)
-
-            # find out names of classes, which are superclasses of our
-            # subject
-            names = set(c.__name__ for c in cls.__mro__ if issubclass(c, Element))
-
-            # find stereotypes that extend out metaclass
-            classes = subject._factory.select(lambda e: e.isKindOf(UML.Class) and e.name in names)
-
-            for class_ in classes:
-                for extension in class_.extension:
-                    stereotype = extension.ownedEnd.type
-                    stereotype_action = ApplyStereotypeAction(stereotype)
-                    register_action(stereotype_action, 'ItemFocus')
-                    stereotype_list.append(stereotype_action.id)
-        return self.popup_menu
 
+    def item_at(self, x, y):
+        return self
 
     def on_subject_notify__appliedStereotype(self, subject, pspec=None):
         if self.subject:
+            self.update_stereotype()
             self.request_update()
 
     def request_update(self):
-        """Placeholder for gaphor.Item's request_update() method.
+        """
+        Placeholder for gaphor.Item's request_update() method.
         """
         pass
 
     def on_subject_notify(self, pspec, notifiers=()):
         SubjectSupport.on_subject_notify(self, pspec, notifiers + ('appliedStereotype',))
 
+
 # vim:sw=4:et:ai
