@@ -2,83 +2,56 @@
 Basic functionality for canvas line based items on a diagram.
 """
 
-import itertools
+from math import atan2, pi
 
 import gaphas
 from gaphas.util import text_extents, text_align
-from gaphas.geometry import Rectangle
 from diagramitem import DiagramItem
 from interfaces import IConnect
 
+from gaphor.diagram.style import get_text_point_at_line, \
+    get_text_point_at_line2, \
+    ALIGN_CENTER, ALIGN_LEFT, ALIGN_RIGHT, ALIGN_TOP
 
 class LineItem(gaphas.Line, DiagramItem):
-    """Base class for diagram lines.
     """
-
-#    __metaclass__ = LineItemMeta
+    Base class for diagram lines.
+    """
 
     def __init__(self, id = None):
         gaphas.Line.__init__(self)
         DiagramItem.__init__(self, id)
         self.fuzziness = 2
-        self._stereotype_bounds = None
 
     head = property(lambda self: self._handles[0])
     tail = property(lambda self: self._handles[-1])
 
-    def update_label(self, context, text):
-        """
-        Update the name label near the middle of the association.
 
-        Returns a geometry.Rectangle object with the boundig box for the label text.
-        """
-        cr = context.cairo
-        handles = self._handles
-        middle = len(handles)/2
-        p1 = handles[middle].pos
-        p2 = handles[middle-1].pos
+    def pre_update(self, context):
+        # first, update stereotype to know its text
+        self.update_stereotype()
 
-        w, h = text_extents(cr, text)
+        #super(LineItem, self).pre_update(context)
+        gaphas.Line.pre_update(self, context)
+        DiagramItem.pre_update(self, context)
 
-        x = p1[0] > p2[0] and w + 5 or -2
-        x = (p1[0] + p2[0]) / 2.0 - x
-        y = p1[1] <= p2[1] and h + 2 or 0
-        y = (p1[1] + p2[1]) / 2.0 - y
 
-        #log.debug('label pos = (%d, %d)' % (x, y))
-        #return x, y, max(x + 10, x + w), max(y + 10, y + h)
-        return Rectangle(x, y, x + max(10, w), y + max(10, h))
+    def post_update(self, context):
+        #super(LineItem, self).update(context)
+        gaphas.Line.post_update(self, context)
+        DiagramItem.post_update(self, context)
 
-    def update_stereotype(self, context):
-        super(LineItem, self).update_stereotype()
-
-        sw, sh = text_extents(context.cairo, self._stereotype)
-
-        handles = self._handles
-        middle = len(handles)/2
-        p1 = handles[middle-1].pos
-        p2 = handles[middle].pos
-
-        x = p1[0] > p2[0] and sw + 2 or -2
-        x = (p1[0] + p2[0]) / 2.0 - x
-        y = p1[1] > p2[1] and -sh or 0
-        y = (p1[1] + p2[1]) / 2.0 - y
-
-        self._stereotype_bounds = Rectangle(x, y, width=sw, height=sh)
-
-    def update(self, context):
-        super(LineItem, self).update(context)
-        cr = context.cairo
-
-        # update stereotype
-        self.update_stereotype(context)
 
     def draw(self, context):
-        super(LineItem, self).draw(context)
-        cr = context.cairo
-        if self._stereotype:
-            text_align(cr, self._stereotype_bounds[0], self._stereotype_bounds[1],
-                       self._stereotype, align_x=1, align_y=1)
+        #super(LineItem, self).draw(context)
+        gaphas.Line.draw(self, context)
+        DiagramItem.draw(self, context)
+
+
+    def point(self, x, y):
+        d1 = gaphas.Line.point(self, x, y)
+        d2 = DiagramItem.point(self, x, y)
+        return min(d1, d2)
 
 
 class DiagramLine(LineItem):
@@ -89,16 +62,6 @@ class DiagramLine(LineItem):
     TODO: put serializability and popup in separate adapters.
     """
 
-    popup_menu = LineItem.popup_menu + (
-        'separator', 
-        'AddSegment',
-        'DeleteSegment',
-        'Orthogonal',
-        'OrthogonalAlignment',
-        'separator',
-        'EditDelete'
-    )
-
     def save (self, save_func):
         LineItem.save(self, save_func)
         save_func('matrix', tuple(self.matrix))
@@ -143,30 +106,84 @@ class DiagramLine(LineItem):
         # First update matrix and solve constraints (NE and SW handle are
         # lazy and are resolved by the constraint solver rather than set
         # directly.
-        self.canvas.update_matrix(self)
-        self.canvas.solver.solve()
+        #self.canvas.update_matrix(self)
+        #self.canvas.solver.solve()
 
         if hasattr(self, '_load_head_connection'):
             adapter = component.queryMultiAdapter((self._load_head_connection, self), IConnect)
             assert adapter, 'No IConnect adapter to connect %s to %s' % (self._load_head_connection, self)
-            h = self.head
 
-            x, y = self.canvas.get_matrix_i2w(self, calculate=True).transform_point(h.x, h.y)
-            x, y = self.canvas.get_matrix_w2i(self._load_head_connection, calculate=True).transform_point(x, y)
-            adapter.connect(h, x, y)
+            adapter.connect(self.head)
             del self._load_head_connection
+
         if hasattr(self, '_load_tail_connection'):
             adapter = component.queryMultiAdapter((self._load_tail_connection, self), IConnect)
             assert adapter, 'No IConnect adapter to connect %s to %s' % (self._load_tail_connection, self)
-            h = self.tail
 
-            x, y = self.canvas.get_matrix_i2w(self, calculate=True).transform_point(h.x, h.y)
-            x, y = self.canvas.get_matrix_w2i(self._load_tail_connection, calculate=True).transform_point(x, y)
-            adapter.connect(h, x, y)
+            adapter.connect(self.tail)
             del self._load_tail_connection
         LineItem.postload(self)
 
 
+    def _get_middle_segment(self):
+        """
+        Get middle line segment.
+        """
+        handles = self._handles
+        m = len(handles) / 2
+        assert m - 1 >= 0 and m < len(handles)
+        return handles[m - 1], handles[m]
+
+
+    def _get_center_pos(self, inverted=False):
+        """
+        Return position in the centre of middle segment of a line. Angle of
+        the middle segment is also returned.
+        """
+        h0, h1 = self._get_middle_segment()
+        pos = (h0.x + h1.x) / 2, (h0.y + h1.y) / 2
+        angle = atan2(h1.y - h0.y, h1.x - h0.x)
+        if inverted:
+            angle += pi
+        return pos, angle
+
+
+    def text_align(self, extents, align, padding, outside):
+        handles = self._handles
+        halign, valign = align
+
+        if halign == ALIGN_LEFT:
+            p1 = handles[0].pos
+            p2 = handles[-1].pos
+            x, y = get_text_point_at_line(extents, p1, p2, align, padding)
+
+        elif halign == ALIGN_CENTER:
+            h0, h1 = self._get_middle_segment()
+            p1 = h0.pos
+            p2 = h1.pos
+            x, y = get_text_point_at_line2(extents, p1, p2, align, padding)
+        elif halign == ALIGN_RIGHT:
+            p1 = handles[-1].pos
+            p2 = handles[-2].pos
+
+            x, y = get_text_point_at_line(extents, p1, p2, align, padding)
+
+        return x, y
+
+
+
+
+class NamedLine(DiagramLine):
+    __namedelement__ = True
+
+    __style__ = {
+            'name-align': (ALIGN_CENTER, ALIGN_TOP),
+            'name-padding': (5, 5, 5, 5),
+            'name-outside': True,
+            'name-align-str': None,
+    }
+
+
 
 class FreeLine(LineItem):
     """
