@@ -6,14 +6,10 @@ ExtensionItem -- Graphical representation of an association.
 # are connected to the same Class, the head_end property is connected to the
 # tail end and visa versa.
 
-from __future__ import generators
-
-from gaphor import resource
 from gaphor import UML
-from gaphor.diagram.diagramitem import DiagramItem
-from gaphor.diagram.diagramline import DiagramLine
+from gaphor.diagram.diagramline import NamedLine
 
-class ExtensionItem(DiagramLine):
+class ExtensionItem(NamedLine):
     """
     ExtensionItem represents associations. 
     An ExtensionItem has two ExtensionEnd items. Each ExtensionEnd item
@@ -23,11 +19,11 @@ class ExtensionItem(DiagramLine):
     __uml__ = UML.Extension
 
     def __init__(self, id=None):
-        DiagramLine.__init__(self, id)
+        NamedLine.__init__(self, id)
 
     def on_subject_notify(self, pspec, notifiers=()):
-        DiagramLine.on_subject_notify(self, pspec,
-                                           notifiers + ('ownedEnd',))
+        NamedLine.on_subject_notify(self, pspec,
+                notifiers + ('ownedEnd',))
 
     def on_subject_notify__ownedEnd(self, subject, pspec):
         self.request_update()
@@ -42,8 +38,5 @@ class ExtensionItem(DiagramLine):
         cr.fill()
         cr.move_to(15, 0)
 
-    def get_popup_menu(self):
-        return self.popup_menu
-
 
 # vim:sw=4:et:ai
