@@ -12,8 +12,6 @@ QueueFull - raised when the queue reaches it's max size and the oldest item
             may not be disposed.
 """
 
-from __future__ import generators
-
 import sys
 import gobject
 import time
@@ -44,7 +42,7 @@ class GIdleThread(object):
     ...         yield x
     >>> t = GIdleThread(counter(123))
     >>> t.start()
-    >>> while gen.is_alive():
+    >>> while t.is_alive():
     ...     main.iteration(False)
     """
 
