@@ -1,5 +1,5 @@
-# vim: sw=4:et
-"""Load and save Gaphor models to Gaphors own XML format.
+"""
+Load and save Gaphor models to Gaphors own XML format.
 
 Three functions are exported:
 load(filename)
@@ -11,9 +11,7 @@ verify(filename)
     we have a valid model, just a valid file).
 """
 
-from __future__ import generators
-
-from cStringIO import StringIO
+from cStringIO import StringIO, InputType
 from xml.sax.saxutils import escape
 import types
 import sys
@@ -25,8 +23,8 @@ import gaphas
 from gaphor import UML
 from gaphor import parser
 from gaphor import diagram
+from gaphor.application import Application
 from gaphor.diagram import items
-from gaphor import resource
 from gaphor.i18n import _
 #from gaphor.misc.xmlwriter import XMLWriter
 
@@ -39,16 +37,16 @@ def save(writer=None, factory=None, status_queue=None):
         if status_queue:
             status_queue(status)
 
-def save_generator(writer=None, factory=None):
-    """Save the current model using @writer, which is a
+def save_generator(writer, factory):
+    """
+    Save the current model using @writer, which is a
     gaphor.misc.xmlwriter.XMLWriter instance (or at least a SAX serializer
     with CDATA support).
     """
-    # Make bool work for Python 2.2
-    bool_ = type(bool(0))
 
     def save_reference(name, value):
-        """Save a value as a reference to another element in the model.
+        """
+        Save a value as a reference to another element in the model.
         This applies to both UML as well as canvas items.
         """
         # Save a reference to the object:
@@ -59,7 +57,8 @@ def save_generator(writer=None, factory=None):
             writer.endElement(name)
 
     def save_collection(name, value):
-        """Save a list of references.
+        """
+        Save a list of references.
         """
         if len(value) > 0:
             writer.startElement(name, {})
@@ -73,7 +72,8 @@ def save_generator(writer=None, factory=None):
             writer.endElement(name)
 
     def save_value(name, value):
-        """Save a value (attribute).
+        """
+        Save a value (attribute).
         If the value is a string, it is saves as a CDATA block.
         """
         if value is not None:
@@ -83,7 +83,7 @@ def save_generator(writer=None, factory=None):
                 writer.startCDATA()
                 writer.characters(value)
                 writer.endCDATA()
-            elif isinstance(value, bool_):
+            elif isinstance(value, bool):
                 # Write booleans as 0/1.
                 writer.characters(str(int(value)))
             else:
@@ -92,7 +92,8 @@ def save_generator(writer=None, factory=None):
             writer.endElement(name)
 
     def save_element(name, value):
-        """Save attributes and references from items in the gaphor.UML module.
+        """
+        Save attributes and references from items in the gaphor.UML module.
         A value may be a primitive (string, int), a gaphor.UML.collection
         (which contains a list of references to other UML elements) or a
         gaphas.Canvas (which contains canvas items).
@@ -110,30 +111,39 @@ def save_generator(writer=None, factory=None):
             save_value(name, value)
 
     def save_canvasitem(name, value, reference=False):
-        """Save attributes and references in a gaphor.diagram.* object.
+        """
+        Save attributes and references in a gaphor.diagram.* object.
         The extra attribute reference can be used to force UML 
         """
         #log.debug('saving canvasitem: %s|%s %s' % (name, value, type(value)))
-        if reference:
-            save_reference(name, value)
-        elif isinstance(value, UML.collection):
+        if isinstance(value, UML.collection) or \
+                (isinstance(value, (list, tuple)) and reference == True):
             save_collection(name, value)
+        elif reference:
+            save_reference(name, value)
         elif isinstance(value, gaphas.Item):
             writer.startElement('item', { 'id': value.id,
                                           'type': value.__class__.__name__ })
             value.save(save_canvasitem)
+
+            # save subitems
+            for child in value.canvas.get_children(value):
+                save_canvasitem(None, child)
+#                writer.startElement('item', { 'id': child.id,
+#                                              'type': kid.__class__.__name__ })
+#                child.save(save_canvasitem)
+#                writer.endElement('item')
+
             writer.endElement('item')
+
         elif isinstance(value, UML.Element):
             save_reference(name, value)
         else:
             save_value(name, value)
 
-    if not factory:
-        factory = resource(UML.ElementFactory)
-
     writer.startDocument()
     writer.startElement('gaphor', { 'version': FILE_FORMAT_VERSION,
-                                    'gaphor-version': resource('Version') })
+                                    'gaphor-version': Application.distribution.version })
 
     size = factory.size()
     n = 0
@@ -143,6 +153,7 @@ def save_generator(writer=None, factory=None):
         writer.startElement(clazz, { 'id': str(e.id) })
         e.save(save_element)
         writer.endElement(clazz)
+
         n += 1
         if n % 25 == 0:
             yield (n * 100) / size
@@ -157,7 +168,8 @@ def load_elements(elements, factory, status_queue=None):
             status_queue(status)
 
 def load_elements_generator(elements, factory, gaphor_version=None):
-    """Load a file and create a model if possible.
+    """
+    Load a file and create a model if possible.
     Exceptions: IOError, ValueError.
     """
     # TODO: restructure loading code, first load model, then add canvas items
@@ -267,21 +279,21 @@ def load_elements_generator(elements, factory, gaphor_version=None):
     # Data model, loaded from file, is updated automatically, so there is
     # no need for special function.
 
+    for d in factory.select(lambda e: isinstance(e, UML.Diagram)):
+        # update_now() is implicitly called when lock is released
+        d.canvas.block_updates = False
+
     # do a postload:
     for id, elem in elements.items():
         yield update_status_queue()
         elem.element.postload()
 
-    # Unlock canvas's for updates
-    for id, elem in elements.items():
-        if isinstance(elem, parser.element) and elem.canvas:
-            elem.element.canvas.block_updates = False
-
     factory.notify_model()
 
 
-def load(filename, factory=None, status_queue=None):
-    """Load a file and create a model if possible.
+def load(filename, factory, status_queue=None):
+    """
+    Load a file and create a model if possible.
     Optionally, a status queue function can be given, to which the
     progress is written (as status_queue(progress)).
     Exceptions: GaphorError.
@@ -290,14 +302,18 @@ def load(filename, factory=None, status_queue=None):
         if status_queue:
             status_queue(status)
 
-def load_generator(filename, factory=None):
-    """Load a file and create a model if possible.
+def load_generator(filename, factory):
+    """
+    Load a file and create a model if possible.
     This function is a generator. It will yield values from 0 to 100 (%)
     to indicate its progression.
 
     Exceptions: GaphorError.
     """
-    log.info('Loading file %s' % os.path.basename(filename))
+    if isinstance(filename, (file, InputType)):
+        log.info('Loading file from file descriptor')
+    else:
+        log.info('Loading file %s' % os.path.basename(filename))
     try:
         # Use the incremental parser and yield the percentage of the file.
         loader = parser.GaphorLoader()
@@ -316,8 +332,6 @@ def load_generator(filename, factory=None):
         raise
 
     try:
-        if not factory:
-            factory = resource(UML.ElementFactory)
         factory.flush()
         gc.collect()
         log.info("Read %d elements from file" % len(elements))
@@ -461,3 +475,5 @@ def version_0_5_2(elements, factory, gaphor_version):
             except Exception, e:
                 log.error('Error while updating Association', e)
 
+
+# vim: sw=4:et
