@@ -2,10 +2,10 @@
 """
 
 import os.path
+import pkg_resources
 from xml.sax import handler
 import gtk
 
-from gaphor import resource
 from gaphor import UML
 from gaphor.parser import ParserException
 
@@ -22,7 +22,7 @@ def get_stock_id(element):
             return _uml_to_stock_id_map[element]
         except KeyError:
             log.warning ('Stock id for %s not found' % element)
-            return STOCK_POINTER
+            return None #STOCK_POINTER
 
 def add_stock_icon(id, icon_dir, icon_files, uml_class=None):
     global _uml_to_stock_id_map
@@ -116,19 +116,23 @@ def load_stock_icons():
     """
     from xml.sax import make_parser
     parser = make_parser()
-    data_dir = resource('DataDir')
-    icon_dir = os.path.join(data_dir, 'pixmaps')
+    icon_dir = os.path.abspath(pkg_resources.resource_filename('gaphor.ui', 'pixmaps'))
+    log.info('Icon dir: %s' % icon_dir)
+    #icon_dir = 'gaphor/data/pixmaps'
     loader = StockIconLoader(icon_dir)
 
     parser.setFeature(handler.feature_namespaces, 1)
     parser.setContentHandler(loader)
 
-    filename = os.path.join(data_dir, 'icons.xml')
-    if os.name == 'nt' and data_dir[1] == ':':
+    filename = pkg_resources.resource_filename('gaphor.ui', 'icons.xml')
+    if os.name == 'nt':
         # Make the filename a full URL
         filename = 'file:' + filename.replace('\\\\', '/')
+    #try:
     parser.parse(filename)
+    #except IOError, e:
+    #    log.error('Unable to load icons', e)
 
-load_stock_icons()
+#load_stock_icons()
 
 # vim:sw=4:et
